@@ -1,23 +1,21 @@
-/* $Id: fileio.cpp 13811 2008-07-23 21:51:25Z rubidium $ */
+/* $Id: fileio.cpp 16273 2009-05-10 21:33:55Z rubidium $ */
 
 /** @file fileio.cpp Standard In/Out file operations */
 
 #include "stdafx.h"
-#include "openttd.h"
-#include "fileio.h"
+#include "fileio_func.h"
 #include "variables.h"
 #include "debug.h"
 #include "fios.h"
-#include "core/alloc_func.hpp"
-#include "core/math_func.hpp"
 #include "string_func.h"
+#include "tar_type.h"
 #ifdef WIN32
 #include <windows.h>
 #else
 #include <pwd.h>
-#include <unistd.h>
 #endif
 #include <sys/stat.h>
+#include <algorithm>
 
 /*************************************************/
 /* FILE IO ROUTINES ******************************/
@@ -27,7 +25,7 @@
 
 struct Fio {
 	byte *buffer, *buffer_end;             ///< position pointer in local buffer and last valid byte of buffer
-	uint32 pos;                            ///< current (system) position in file
+	size_t pos;                            ///< current (system) position in file
 	FILE *cur_fh;                          ///< current file handle
 	const char *filename;                  ///< current filename
 	FILE *handles[MAX_FILE_SLOTS];         ///< array of file handles we can have open
@@ -43,9 +41,9 @@ struct Fio {
 static Fio _fio;
 
 /* Get current position in file */
-uint32 FioGetPos()
+size_t FioGetPos()
 {
-	return _fio.pos + (_fio.buffer - _fio.buffer_start) - FIO_BUFFER_SIZE;
+	return _fio.pos + (_fio.buffer - _fio.buffer_end);
 }
 
 const char *FioGetFilename(uint8 slot)
@@ -53,7 +51,7 @@ const char *FioGetFilename(uint8 slot)
 	return _fio.shortnames[slot];
 }
 
-void FioSeekTo(uint32 pos, int mode)
+void FioSeekTo(size_t pos, int mode)
 {
 	if (mode == SEEK_CUR) pos += FioGetPos();
 	_fio.buffer = _fio.buffer_end = _fio.buffer_start + FIO_BUFFER_SIZE;
@@ -74,7 +72,7 @@ static void FioRestoreFile(int slot)
 #endif /* LIMITED_FDS */
 
 /* Seek to a file and a position */
-void FioSeekToFile(uint8 slot, uint32 pos)
+void FioSeekToFile(uint8 slot, size_t pos)
 {
 	FILE *f;
 #if defined(LIMITED_FDS)
@@ -91,8 +89,12 @@ void FioSeekToFile(uint8 slot, uint32 pos)
 byte FioReadByte()
 {
 	if (_fio.buffer == _fio.buffer_end) {
-		_fio.pos += FIO_BUFFER_SIZE;
-		fread(_fio.buffer = _fio.buffer_start, 1, FIO_BUFFER_SIZE, _fio.cur_fh);
+		_fio.buffer = _fio.buffer_start;
+		size_t size = fread(_fio.buffer, 1, FIO_BUFFER_SIZE, _fio.cur_fh);
+		_fio.pos += size;
+		_fio.buffer_end = _fio.buffer_start + size;
+
+		if (size == 0) return 0;
 	}
 	return *_fio.buffer++;
 }
@@ -121,11 +123,10 @@ uint32 FioReadDword()
 	return (FioReadWord() << 16) | b;
 }
 
-void FioReadBlock(void *ptr, uint size)
+void FioReadBlock(void *ptr, size_t size)
 {
 	FioSeekTo(FioGetPos(), SEEK_SET);
-	_fio.pos += size;
-	fread(ptr, 1, size, _fio.cur_fh);
+	_fio.pos += fread(ptr, 1, size, _fio.cur_fh);
 }
 
 static inline void FioCloseFile(int slot)
@@ -183,7 +184,7 @@ void FioOpenFile(int slot, const char *filename)
 	FioFreeHandle();
 #endif /* LIMITED_FDS */
 	f = FioFOpenFile(filename);
-	if (f == NULL) error("Cannot open file '%s'", filename);
+	if (f == NULL) usererror("Cannot open file '%s'", filename);
 	uint32 pos = ftell(f);
 
 	FioCloseFile(slot); // if file was opened before, close it
@@ -212,13 +213,18 @@ const char *_subdirs[NUM_SUBDIRS] = {
 	"scenario" PATHSEP "heightmap" PATHSEP,
 	"gm" PATHSEP,
 	"data" PATHSEP,
-	"lang" PATHSEP
+	"lang" PATHSEP,
+	"ai" PATHSEP,
+	"ai" PATHSEP "library" PATHSEP,
 };
 
 const char *_searchpaths[NUM_SEARCHPATHS];
 TarList _tar_list;
 TarFileList _tar_filelist;
 
+typedef std::map<std::string, std::string> TarLinkList;
+static TarLinkList _tar_linklist; ///< List of directory links
+
 /**
  * Check whether the given file exists
  * @param filename the file to try for existance
@@ -303,7 +309,7 @@ FILE *FioFOpenFileSp(const char *filename, const char *mode, Searchpath sp, Subd
 	char buf[MAX_PATH];
 
 	if (subdir == NO_DIRECTORY) {
-		ttd_strlcpy(buf, filename, lengthof(buf));
+		strecpy(buf, filename, lastof(buf));
 	} else {
 		snprintf(buf, lengthof(buf), "%s%s%s", _searchpaths[sp], _subdirs[subdir], filename);
 	}
@@ -352,12 +358,32 @@ FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir,
 	}
 
 	/* We can only use .tar in case of data-dir, and read-mode */
-	if (f == NULL && subdir == DATA_DIR && mode[0] == 'r') {
+	if (f == NULL && mode[0] == 'r') {
+		static const uint MAX_RESOLVED_LENGTH = 2 * (100 + 100 + 155) + 1; // Enough space to hold two filenames plus link. See 'TarHeader'.
+		char resolved_name[MAX_RESOLVED_LENGTH];
+
 		/* Filenames in tars are always forced to be lowercase */
-		char *lcfilename = strdup(filename);
-		strtolower(lcfilename);
-		TarFileList::iterator it = _tar_filelist.find(lcfilename);
-		free(lcfilename);
+		strecpy(resolved_name, filename, lastof(resolved_name));
+		strtolower(resolved_name);
+
+		size_t resolved_len = strlen(resolved_name);
+
+		/* Resolve ONE directory link */
+		for (TarLinkList::iterator link = _tar_linklist.begin(); link != _tar_linklist.end(); link++) {
+			const std::string &src = link->first;
+			size_t len = src.length();
+			if (resolved_len >= len && resolved_name[len - 1] == PATHSEPCHAR && strncmp(src.c_str(), resolved_name, len) == 0) {
+				/* Apply link */
+				char resolved_name2[MAX_RESOLVED_LENGTH];
+				const std::string &dest = link->second;
+				strecpy(resolved_name2, &(resolved_name[len]), lastof(resolved_name2));
+				strecpy(resolved_name, dest.c_str(), lastof(resolved_name));
+				strecpy(&(resolved_name[dest.length()]), resolved_name2, lastof(resolved_name));
+				break; // Only resolve one level
+			}
+		}
+
+		TarFileList::iterator it = _tar_filelist.find(resolved_name);
 		if (it != _tar_filelist.end()) {
 			f = FioFOpenFileTar(&((*it).second), filesize);
 		}
@@ -430,7 +456,7 @@ char *BuildWithFullPath(const char *dir)
 
 	/* Add absolute path */
 	if (s == NULL || dest != s) {
-		getcwd(dest, MAX_PATH);
+		if (getcwd(dest, MAX_PATH) == NULL) *dest = '\0';
 		AppendPathSeparator(dest, MAX_PATH);
 		ttd_strlcat(dest, dir, MAX_PATH);
 	}
@@ -439,7 +465,56 @@ char *BuildWithFullPath(const char *dir)
 	return dest;
 }
 
-static bool TarListAddFile(const char *filename)
+const char *FioTarFirstDir(const char *tarname)
+{
+	TarList::iterator it = _tar_list.find(tarname);
+	if (it == _tar_list.end()) return NULL;
+	return (*it).second.dirname;
+}
+
+static void TarAddLink(const std::string &srcParam, const std::string &destParam)
+{
+	std::string src = srcParam;
+	std::string dest = destParam;
+	/* Tar internals assume lowercase */
+	std::transform(src.begin(), src.end(), src.begin(), tolower);
+	std::transform(dest.begin(), dest.end(), dest.begin(), tolower);
+
+	TarFileList::iterator dest_file = _tar_filelist.find(dest);
+	if (dest_file != _tar_filelist.end()) {
+		/* Link to file. Process the link like the destination file. */
+		_tar_filelist.insert(TarFileList::value_type(src, dest_file->second));
+	} else {
+		/* Destination file not found. Assume 'link to directory'
+		 * Append PATHSEPCHAR to 'src' and 'dest' if needed */
+		const std::string src_path = ((*src.rbegin() == PATHSEPCHAR) ? src : src + PATHSEPCHAR);
+		const std::string dst_path = (dest.length() == 0 ? "" : ((*dest.rbegin() == PATHSEPCHAR) ? dest : dest + PATHSEPCHAR));
+		_tar_linklist.insert(TarLinkList::value_type(src_path, dst_path));
+	}
+}
+
+void FioTarAddLink(const char *src, const char *dest)
+{
+	TarAddLink(src, dest);
+}
+
+/**
+ * Simplify filenames from tars.
+ * Replace '/' by PATHSEPCHAR, and force 'name' to lowercase.
+ * @param name Filename to process.
+ */
+static void SimplifyFileName(char *name)
+{
+	/* Force lowercase */
+	strtolower(name);
+
+	/* Tar-files always have '/' path-seperator, but we want our PATHSEPCHAR */
+#if (PATHSEPCHAR != '/')
+	for (char *n = name; *n != '\0'; n++) if (*n == '/') *n = PATHSEPCHAR;
+#endif
+}
+
+bool TarListAddFile(const char *filename)
 {
 	/* The TAR-header, repeated for every file */
 	typedef struct TarHeader {
@@ -472,11 +547,16 @@ static bool TarListAddFile(const char *filename)
 
 	const char *dupped_filename = strdup(filename);
 	_tar_list[filename].filename = dupped_filename;
+	_tar_list[filename].dirname = NULL;
+
+	TarLinkList links; ///< Temporary list to collect links
 
 	TarHeader th;
 	char buf[sizeof(th.name) + 1], *end;
 	char name[sizeof(th.prefix) + 1 + sizeof(th.name) + 1];
-	int num = 0, pos = 0;
+	char link[sizeof(th.linkname) + 1];
+	char dest[sizeof(th.prefix) + 1 + sizeof(th.name) + 1 + 1 + sizeof(th.linkname) + 1];
+	size_t num = 0, pos = 0;
 
 	/* Make a char of 512 empty bytes */
 	char empty[512];
@@ -497,7 +577,7 @@ static bool TarListAddFile(const char *filename)
 		}
 
 		name[0] = '\0';
-		int len = 0;
+		size_t len = 0;
 
 		/* The prefix contains the directory-name */
 		if (th.prefix[0] != '\0') {
@@ -515,26 +595,106 @@ static bool TarListAddFile(const char *filename)
 		/* Calculate the size of the file.. for some strange reason this is stored as a string */
 		memcpy(buf, th.size, sizeof(th.size));
 		buf[sizeof(th.size)] = '\0';
-		int skip = strtol(buf, &end, 8);
+		size_t skip = strtoul(buf, &end, 8);
 
-		/* 0 byte sized files can be skipped (dirs, symlinks, ..) */
-		if (skip == 0) continue;
+		switch (th.typeflag) {
+			case '\0':
+			case '0': { // regular file
+				/* Ignore empty files */
+				if (skip == 0) break;
 
-		/* Store this entry in the list */
-		TarFileListEntry entry;
-		entry.tar_filename = dupped_filename;
-		entry.size         = skip;
-		entry.position     = pos;
-		/* Force lowercase */
-		strtolower(name);
+				if (strlen(name) == 0) break;
 
-		/* Tar-files always have '/' path-seperator, but we want our PATHSEPCHAR */
-#if (PATHSEPCHAR != '/')
-		for (char *n = name; *n != '\0'; n++) if (*n == '/') *n = PATHSEPCHAR;
-#endif
+				/* Store this entry in the list */
+				TarFileListEntry entry;
+				entry.tar_filename = dupped_filename;
+				entry.size         = skip;
+				entry.position     = pos;
 
-		DEBUG(misc, 6, "Found file in tar: %s (%d bytes, %d offset)", name, skip, pos);
-		if (_tar_filelist.insert(TarFileList::value_type(name, entry)).second) num++;
+				/* Convert to lowercase and our PATHSEPCHAR */
+				SimplifyFileName(name);
+
+				DEBUG(misc, 6, "Found file in tar: %s (" PRINTF_SIZE " bytes, " PRINTF_SIZE " offset)", name, skip, pos);
+				if (_tar_filelist.insert(TarFileList::value_type(name, entry)).second) num++;
+
+				break;
+			}
+
+			case '1': // hard links
+			case '2': { // symbolic links
+				/* Copy the destination of the link in a safe way at the end of 'linkname' */
+				memcpy(link, th.linkname, sizeof(th.linkname));
+				link[sizeof(th.linkname)] = '\0';
+
+				if (strlen(name) == 0 || strlen(link) == 0) break;
+
+				/* Convert to lowercase and our PATHSEPCHAR */
+				SimplifyFileName(name);
+				SimplifyFileName(link);
+
+				/* Only allow relative links */
+				if (link[0] == PATHSEPCHAR) {
+					DEBUG(misc, 1, "Ignoring absolute link in tar: %s -> %s", name, link);
+					break;
+				}
+
+				/* Process relative path.
+				 * Note: The destination of links must not contain any directory-links. */
+				strecpy(dest, name, lastof(dest));
+				char *destpos = strrchr(dest, PATHSEPCHAR);
+				if (destpos == NULL) destpos = dest;
+				*destpos = '\0';
+
+				char *pos = link;
+				while (*pos != '\0') {
+					char *next = strchr(link, PATHSEPCHAR);
+					if (next == NULL) next = pos + strlen(pos);
+
+					/* Skip '.' (current dir) */
+					if (next != pos + 1 || pos[0] != '.') {
+						if (next == pos + 2 && pos[0] == '.' && pos[1] == '.') {
+							/* level up */
+							if (dest[0] == '\0') {
+								DEBUG(misc, 1, "Ignoring link pointing outside of data directory: %s -> %s", name, link);
+								break;
+							}
+
+							/* Truncate 'dest' after last PATHSEPCHAR.
+							 * This assumes, that the truncated part is a real directory and not a link */
+							destpos = strrchr(dest, PATHSEPCHAR);
+							if (destpos == NULL) destpos = dest;
+						} else {
+							/* Append at end of 'dest' */
+							if (destpos != dest) *(destpos++) = PATHSEPCHAR;
+							strncpy(destpos, pos, next - pos); // Safe as we do '\0'-termination ourselves
+							destpos += next - pos;
+						}
+						*destpos = '\0';
+					}
+
+					pos = next;
+				}
+
+				/* Store links in temporary list */
+				DEBUG(misc, 6, "Found link in tar: %s -> %s", name, dest);
+				links.insert(TarLinkList::value_type(name, dest));
+
+				break;
+			}
+
+			case '5': // directory
+				/* Convert to lowercase and our PATHSEPCHAR */
+				SimplifyFileName(name);
+
+				/* Store the first directory name we detect */
+				DEBUG(misc, 6, "Found dir in tar: %s", name);
+				if (_tar_list[filename].dirname == NULL) _tar_list[filename].dirname = strdup(name);
+				break;
+
+			default:
+				/* Ignore other types */
+				break;
+		}
 
 		/* Skip to the next block.. */
 		skip = Align(skip, 512);
@@ -542,13 +702,28 @@ static bool TarListAddFile(const char *filename)
 		pos += skip;
 	}
 
-	DEBUG(misc, 1, "Found tar '%s' with %d new files", filename, num);
+	DEBUG(misc, 1, "Found tar '%s' with " PRINTF_SIZE " new files", filename, num);
 	fclose(f);
 
+	/* Resolve file links and store directory links.
+	 * We restrict usage of links to two cases:
+	 *  1) Links to directories:
+	 *      Both the source path and the destination path must NOT contain any further links.
+	 *      When resolving files at most one directory link is resolved.
+	 *  2) Links to files:
+	 *      The destination path must NOT contain any links.
+	 *      The source path may contain one directory link.
+	 */
+	for (TarLinkList::iterator link = links.begin(); link != links.end(); link++) {
+		const std::string &src = link->first;
+		const std::string &dest = link->second;
+		TarAddLink(src, dest);
+	}
+
 	return true;
 }
 
-static int ScanPathForTarFiles(const char *path, int basepath_length)
+static int ScanPathForTarFiles(const char *path, size_t basepath_length)
 {
 	extern bool FiosIsValidFile(const char *path, const struct dirent *ent, struct stat *sb);
 
@@ -567,12 +742,12 @@ static int ScanPathForTarFiles(const char *path, int basepath_length)
 
 		snprintf(filename, lengthof(filename), "%s%s", path, d_name);
 
-		if (sb.st_mode & S_IFDIR) {
+		if (S_ISDIR(sb.st_mode)) {
 			/* Directory */
 			if (strcmp(d_name, ".") == 0 || strcmp(d_name, "..") == 0) continue;
 			AppendPathSeparator(filename, lengthof(filename));
 			num += ScanPathForTarFiles(filename, basepath_length);
-		} else if (sb.st_mode & S_IFREG) {
+		} else if (S_ISREG(sb.st_mode)) {
 			/* File */
 			char *ext = strrchr(filename, '.');
 
@@ -598,6 +773,12 @@ void ScanForTarFiles()
 	FOR_ALL_SEARCHPATHS(sp) {
 		FioAppendDirectory(path, MAX_PATH, sp, DATA_DIR);
 		num += ScanPathForTarFiles(path, strlen(path));
+		FioAppendDirectory(path, MAX_PATH, sp, AI_DIR);
+		num += ScanPathForTarFiles(path, strlen(path));
+		FioAppendDirectory(path, MAX_PATH, sp, AI_LIBRARY_DIR);
+		num += ScanPathForTarFiles(path, strlen(path));
+		FioAppendDirectory(path, MAX_PATH, sp, SCENARIO_DIR);
+		num += ScanPathForTarFiles(path, strlen(path));
 	}
 	DEBUG(misc, 1, "Scan complete, found %d files", num);
 }
@@ -626,10 +807,14 @@ void ChangeWorkingDirectory(const char *exe)
 
 	if (app_bundle != NULL) app_bundle[0] = '\0';
 #endif /* WITH_COCOA */
-	char *s = strrchr(exe, PATHSEPCHAR);
+	char *s = (char*)strrchr(exe, PATHSEPCHAR);
 	if (s != NULL) {
 		*s = '\0';
-		chdir(exe);
+#if defined(__DJGPP__)
+		/* If we want to go to the root, we can't use cd C:, but we must use '/' */
+		if (s[-1] == ':') chdir("/");
+#endif
+		if (chdir(exe) != 0) DEBUG(misc, 0, "Directory with the binary does not exist?");
 		*s = PATHSEPCHAR;
 	}
 #ifdef WITH_COCOA
@@ -644,7 +829,7 @@ void ChangeWorkingDirectory(const char *exe)
 void DetermineBasePaths(const char *exe)
 {
 	char tmp[MAX_PATH];
-#if defined(__MORPHOS__) || defined(__AMIGA__) || !defined(WITH_PERSONAL_DIR)
+#if defined(__MORPHOS__) || defined(__AMIGA__) || defined(DOS) || defined(OS2) || !defined(WITH_PERSONAL_DIR)
 	_searchpaths[SP_PERSONAL_DIR] = NULL;
 #else
 	const char *homedir = getenv("HOME");
@@ -671,18 +856,18 @@ void DetermineBasePaths(const char *exe)
 #if defined(__MORPHOS__) || defined(__AMIGA__)
 	_searchpaths[SP_WORKING_DIR] = NULL;
 #else
-	getcwd(tmp, MAX_PATH);
+	if (getcwd(tmp, MAX_PATH) == NULL) *tmp = '\0';
 	AppendPathSeparator(tmp, MAX_PATH);
 	_searchpaths[SP_WORKING_DIR] = strdup(tmp);
 #endif
 
 	/* Change the working directory to that one of the executable */
 	ChangeWorkingDirectory((char*)exe);
-	getcwd(tmp, MAX_PATH);
+	if (getcwd(tmp, MAX_PATH) == NULL) *tmp = '\0';
 	AppendPathSeparator(tmp, MAX_PATH);
 	_searchpaths[SP_BINARY_DIR] = strdup(tmp);
 
-#if defined(__MORPHOS__) || defined(__AMIGA__)
+#if defined(__MORPHOS__) || defined(__AMIGA__) || defined(DOS) || defined(OS2)
 	_searchpaths[SP_INSTALLATION_DIR] = NULL;
 #else
 	snprintf(tmp, MAX_PATH, "%s", GLOBAL_DATA_DIR);
@@ -695,8 +880,6 @@ extern void cocoaSetApplicationBundleDir();
 #else
 	_searchpaths[SP_APPLICATION_BUNDLE_DIR] = NULL;
 #endif
-
-	ScanForTarFiles();
 }
 #endif /* defined(WIN32) || defined(WINCE) */
 
@@ -752,19 +935,43 @@ void DeterminePaths(const char *exe)
 	_highscore_file = str_fmt("%shs.dat", _personal_dir);
 	_log_file = str_fmt("%sopenttd.log",  _personal_dir);
 
-	char *save_dir     = str_fmt("%s%s", _personal_dir, FioGetSubdirectory(SAVE_DIR));
-	char *autosave_dir = str_fmt("%s%s", _personal_dir, FioGetSubdirectory(AUTOSAVE_DIR));
-
 	/* Make the necessary folders */
 #if !defined(__MORPHOS__) && !defined(__AMIGA__) && defined(WITH_PERSONAL_DIR)
 	FioCreateDirectory(_personal_dir);
 #endif
 
-	FioCreateDirectory(save_dir);
-	FioCreateDirectory(autosave_dir);
+	static const Subdirectory default_subdirs[] = {
+		SAVE_DIR, AUTOSAVE_DIR, SCENARIO_DIR, HEIGHTMAP_DIR
+	};
 
-	free(save_dir);
-	free(autosave_dir);
+	for (uint i = 0; i < lengthof(default_subdirs); i++) {
+		char *dir = str_fmt("%s%s", _personal_dir, FioGetSubdirectory(default_subdirs[i]));
+		FioCreateDirectory(dir);
+		free(dir);
+	}
+
+	/* If we have network we make a directory for the autodownloading of content */
+	_searchpaths[SP_AUTODOWNLOAD_DIR] = str_fmt("%s%s", _personal_dir, "content_download" PATHSEP);
+#ifdef ENABLE_NETWORK
+	FioCreateDirectory(_searchpaths[SP_AUTODOWNLOAD_DIR]);
+
+	/* Create the directory for each of the types of content */
+	const Subdirectory dirs[] = { SCENARIO_DIR, HEIGHTMAP_DIR, DATA_DIR, AI_DIR, AI_LIBRARY_DIR };
+	for (uint i = 0; i < lengthof(dirs); i++) {
+		char *tmp = str_fmt("%s%s", _searchpaths[SP_AUTODOWNLOAD_DIR], FioGetSubdirectory(dirs[i]));
+		FioCreateDirectory(tmp);
+		free(tmp);
+	}
+#else /* ENABLE_NETWORK */
+	/* If we don't have networking, we don't need to make the directory. But
+	 * if it exists we keep it, otherwise remove it from the search paths. */
+	if (!FileExists(_searchpaths[SP_AUTODOWNLOAD_DIR]))  {
+		free((void*)_searchpaths[SP_AUTODOWNLOAD_DIR]);
+		_searchpaths[SP_AUTODOWNLOAD_DIR] = NULL;
+	}
+#endif /* ENABLE_NETWORK */
+
+	ScanForTarFiles();
 }
 
 /**
@@ -787,20 +994,17 @@ void SanitizeFilename(char *filename)
 
 void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 {
-	FILE *in;
-	byte *mem;
-	size_t len;
-
-	in = fopen(filename, "rb");
+	FILE *in = fopen(filename, "rb");
 	if (in == NULL) return NULL;
 
 	fseek(in, 0, SEEK_END);
-	len = ftell(in);
+	size_t len = ftell(in);
 	fseek(in, 0, SEEK_SET);
-	if (len > maxsize || (mem = MallocT<byte>(len + 1)) == NULL) {
+	if (len > maxsize) {
 		fclose(in);
 		return NULL;
 	}
+	byte *mem = MallocT<byte>(len + 1);
 	mem[len] = 0;
 	if (fread(mem, len, 1, in) != 1) {
 		fclose(in);
@@ -812,3 +1016,122 @@ void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 	*lenp = len;
 	return mem;
 }
+
+
+/**
+ * Scan a single directory (and recursively it's children) and add
+ * any graphics sets that are found.
+ * @param extension       the extension of files to search for.
+ * @param path            full path we're currently at
+ * @param basepath_length from where in the path are we 'based' on the search path
+ */
+static uint ScanPath(FileScanner *fs, const char *extension, const char *path, size_t basepath_length, bool recursive)
+{
+	extern bool FiosIsValidFile(const char *path, const struct dirent *ent, struct stat *sb);
+
+	uint num = 0;
+	struct stat sb;
+	struct dirent *dirent;
+	DIR *dir;
+
+	if (path == NULL || (dir = ttd_opendir(path)) == NULL) return 0;
+
+	while ((dirent = readdir(dir)) != NULL) {
+		const char *d_name = FS2OTTD(dirent->d_name);
+		char filename[MAX_PATH];
+
+		if (!FiosIsValidFile(path, dirent, &sb)) continue;
+
+		snprintf(filename, lengthof(filename), "%s%s", path, d_name);
+
+		if (S_ISDIR(sb.st_mode)) {
+			/* Directory */
+			if (!recursive) continue;
+			if (strcmp(d_name, ".") == 0 || strcmp(d_name, "..") == 0) continue;
+			AppendPathSeparator(filename, lengthof(filename));
+			num += ScanPath(fs, extension, filename, basepath_length, recursive);
+		} else if (S_ISREG(sb.st_mode)) {
+			/* File */
+			if (extension != NULL) {
+				char *ext = strrchr(filename, '.');
+
+				/* If no extension or extension isn't .grf, skip the file */
+				if (ext == NULL) continue;
+				if (strcasecmp(ext, extension) != 0) continue;
+			}
+
+			if (fs->AddFile(filename, basepath_length)) num++;
+		}
+	}
+
+	closedir(dir);
+
+	return num;
+}
+
+/**
+ * Scan the given tar and add graphics sets when it finds one.
+ * @param extension the extension of files to search for.
+ * @param tar       the tar to search in.
+ */
+static uint ScanTar(FileScanner *fs, const char *extension, TarFileList::iterator tar)
+{
+	uint num = 0;
+	const char *filename = (*tar).first.c_str();
+
+	if (extension != NULL) {
+		const char *ext = strrchr(filename, '.');
+
+		/* If no extension or extension isn't .grf, skip the file */
+		if (ext == NULL) return false;
+		if (strcasecmp(ext, extension) != 0) return false;
+	}
+
+	if (fs->AddFile(filename, 0)) num++;
+
+	return num;
+}
+
+/**
+ * Scan for files with the given extention in the given search path.
+ * @param extension the extension of files to search for.
+ * @param sd        the sub directory to search in.
+ * @param tars      whether to search in the tars too.
+ * @return the number of found files, i.e. the number of times that
+ *         AddFile returned true.
+ */
+uint FileScanner::Scan(const char *extension, Subdirectory sd, bool tars, bool recursive)
+{
+	Searchpath sp;
+	char path[MAX_PATH];
+	TarFileList::iterator tar;
+	uint num = 0;
+
+	FOR_ALL_SEARCHPATHS(sp) {
+		FioAppendDirectory(path, MAX_PATH, sp, sd);
+		num += ScanPath(this, extension, path, strlen(path), recursive);
+	}
+
+	if (tars) {
+		FOR_ALL_TARS(tar) {
+			num += ScanTar(this, extension, tar);
+		}
+	}
+
+	return num;
+}
+
+/**
+ * Scan for files with the given extention in the given search path.
+ * @param extension the extension of files to search for.
+ * @param directory the sub directory to search in.
+ * @return the number of found files, i.e. the number of times that
+ *         AddFile returned true.
+ */
+uint FileScanner::Scan(const char *extension, const char *directory, bool recursive)
+{
+	char path[MAX_PATH];
+	strecpy(path, directory, lastof(path));
+	AppendPathSeparator(path, lengthof(path));
+	return ScanPath(this, extension, path, strlen(path), recursive);
+}
