@@ -1,29 +1,27 @@
-/* $Id: landscape.cpp 13827 2008-07-25 19:54:14Z rubidium $ */
+/* $Id: landscape.cpp 15903 2009-03-30 23:15:05Z rubidium $ */
+
+/** @file landscape.cpp Functions related to the landscape (slopes etc.). */
+
+/** @defgroup SnowLineGroup Snowline functions and data structures */
 
 #include "stdafx.h"
-#include "openttd.h"
-#include "bridge_map.h"
 #include "heightmap.h"
 #include "clear_map.h"
 #include "spritecache.h"
-#include <stdarg.h>
 #include "viewport_func.h"
 #include "command_func.h"
 #include "landscape.h"
 #include "variables.h"
 #include "void_map.h"
-#include "water_map.h"
 #include "tgp.h"
 #include "genworld.h"
-#include "tile_cmd.h"
-#include "core/alloc_func.hpp"
 #include "fios.h"
-#include "window_func.h"
 #include "functions.h"
 #include "date_func.h"
-#include "vehicle_func.h"
-#include "settings_type.h"
 #include "water.h"
+#include "effectvehicle_func.h"
+#include "landscape_type.h"
+#include "settings_type.h"
 
 #include "table/sprites.h"
 
@@ -40,18 +38,21 @@ extern const TileTypeProcs
 	_tile_type_tunnelbridge_procs,
 	_tile_type_unmovable_procs;
 
+/** Tile callback functions for each type of tile.
+ * @ingroup TileCallbackGroup
+ * @see TileType */
 const TileTypeProcs * const _tile_type_procs[16] = {
-	&_tile_type_clear_procs,
-	&_tile_type_rail_procs,
-	&_tile_type_road_procs,
-	&_tile_type_town_procs,
-	&_tile_type_trees_procs,
-	&_tile_type_station_procs,
-	&_tile_type_water_procs,
-	&_tile_type_dummy_procs,
-	&_tile_type_industry_procs,
-	&_tile_type_tunnelbridge_procs,
-	&_tile_type_unmovable_procs,
+	&_tile_type_clear_procs,        ///< Callback functions for MP_CLEAR tiles
+	&_tile_type_rail_procs,         ///< Callback functions for MP_RAILWAY tiles
+	&_tile_type_road_procs,         ///< Callback functions for MP_ROAD tiles
+	&_tile_type_town_procs,         ///< Callback functions for MP_HOUSE tiles
+	&_tile_type_trees_procs,        ///< Callback functions for MP_TREES tiles
+	&_tile_type_station_procs,      ///< Callback functions for MP_STATION tiles
+	&_tile_type_water_procs,        ///< Callback functions for MP_WATER tiles
+	&_tile_type_dummy_procs,        ///< Callback functions for MP_VOID tiles
+	&_tile_type_industry_procs,     ///< Callback functions for MP_INDUSTRY tiles
+	&_tile_type_tunnelbridge_procs, ///< Callback functions for MP_TUNNELBRIDGE tiles
+	&_tile_type_unmovable_procs,    ///< Callback functions for MP_UNMOVABLE tiles
 };
 
 /* landscape slope => sprite */
@@ -60,10 +61,17 @@ const byte _tileh_to_sprite[32] = {
 	0, 0, 0, 0, 0, 0, 0, 16, 0, 0,  0, 17,  0, 15, 18, 0,
 };
 
+/**
+ * Description of the snow line throughout the year.
+ *
+ * If it is \c NULL, a static snowline height is used, as set by \c _settings_game.game_creation.snow_line.
+ * Otherwise it points to a table loaded from a newGRF file, that describes the variable snowline
+ * @ingroup SnowLineGroup
+ * @see GetSnowLine() GameCreationSettings */
 SnowLine *_snow_line = NULL;
 
 /**
- * Applys a foundation to a slope.
+ * Applies a foundation to a slope.
  *
  * @pre      Foundation and slope must be valid combined.
  * @param f  The #Foundation.
@@ -72,7 +80,6 @@ SnowLine *_snow_line = NULL;
  */
 uint ApplyFoundationToSlope(Foundation f, Slope *s)
 {
-
 	if (!IsFoundation(f)) return 0;
 
 	if (IsLeveledFoundation(f)) {
@@ -117,6 +124,13 @@ uint ApplyFoundationToSlope(Foundation f, Slope *s)
 }
 
 
+/**
+ * Determines height at given coordinate of a slope
+ * @param x x coordinate
+ * @param y y coordinate
+ * @param corners slope to examine
+ * @return height of given point of given slope
+ */
 uint GetPartialZ(int x, int y, Slope corners)
 {
 	if (IsHalftileSlope(corners)) {
@@ -144,91 +158,99 @@ uint GetPartialZ(int x, int y, Slope corners)
 	int z = 0;
 
 	switch (RemoveHalftileSlope(corners)) {
-	case SLOPE_W:
-		if (x - y >= 0)
-			z = (x - y) >> 1;
-		break;
+		case SLOPE_W:
+			if (x - y >= 0) {
+				z = (x - y) >> 1;
+			}
+			break;
 
-	case SLOPE_S:
-		y ^= 0xF;
-		if ( (x - y) >= 0)
-			z = (x - y) >> 1;
-		break;
+		case SLOPE_S:
+			y ^= 0xF;
+			if ((x - y) >= 0) {
+				z = (x - y) >> 1;
+			}
+			break;
 
-	case SLOPE_SW:
-		z = (x >> 1) + 1;
-		break;
+		case SLOPE_SW:
+			z = (x >> 1) + 1;
+			break;
 
-	case SLOPE_E:
-		if (y - x >= 0)
-			z = (y - x) >> 1;
-		break;
+		case SLOPE_E:
+			if (y - x >= 0) {
+				z = (y - x) >> 1;
+			}
+			break;
 
-	case SLOPE_EW:
-	case SLOPE_NS:
-	case SLOPE_ELEVATED:
-		z = 4;
-		break;
+		case SLOPE_EW:
+		case SLOPE_NS:
+		case SLOPE_ELEVATED:
+			z = 4;
+			break;
 
-	case SLOPE_SE:
-		z = (y >> 1) + 1;
-		break;
+		case SLOPE_SE:
+			z = (y >> 1) + 1;
+			break;
 
-	case SLOPE_WSE:
-		z = 8;
-		y ^= 0xF;
-		if (x - y < 0)
-			z += (x - y) >> 1;
-		break;
+		case SLOPE_WSE:
+			z = 8;
+			y ^= 0xF;
+			if (x - y < 0) {
+				z += (x - y) >> 1;
+			}
+			break;
 
-	case SLOPE_N:
-		y ^= 0xF;
-		if (y - x >= 0)
-			z = (y - x) >> 1;
-		break;
+		case SLOPE_N:
+			y ^= 0xF;
+			if (y - x >= 0) {
+				z = (y - x) >> 1;
+			}
+			break;
 
-	case SLOPE_NW:
-		z = (y ^ 0xF) >> 1;
-		break;
+		case SLOPE_NW:
+			z = (y ^ 0xF) >> 1;
+			break;
 
-	case SLOPE_NWS:
-		z = 8;
-		if (x - y < 0)
-			z += (x - y) >> 1;
-		break;
+		case SLOPE_NWS:
+			z = 8;
+			if (x - y < 0) {
+				z += (x - y) >> 1;
+			}
+			break;
 
-	case SLOPE_NE:
-		z = (x ^ 0xF) >> 1;
-		break;
+		case SLOPE_NE:
+			z = (x ^ 0xF) >> 1;
+			break;
 
-	case SLOPE_ENW:
-		z = 8;
-		y ^= 0xF;
-		if (y - x < 0)
-			z += (y - x) >> 1;
-		break;
+		case SLOPE_ENW:
+			z = 8;
+			y ^= 0xF;
+			if (y - x < 0) {
+				z += (y - x) >> 1;
+			}
+			break;
 
-	case SLOPE_SEN:
-		z = 8;
-		if (y - x < 0)
-			z += (y - x) >> 1;
-		break;
+		case SLOPE_SEN:
+			z = 8;
+			if (y - x < 0) {
+				z += (y - x) >> 1;
+			}
+			break;
 
-	case SLOPE_STEEP_S:
-		z = 1 + ((x + y) >> 1);
-		break;
+		case SLOPE_STEEP_S:
+			z = 1 + ((x + y) >> 1);
+			break;
 
-	case SLOPE_STEEP_W:
-		z = 1 + ((x + (y ^ 0xF)) >> 1);
-		break;
+		case SLOPE_STEEP_W:
+			z = 1 + ((x + (y ^ 0xF)) >> 1);
+			break;
 
-	case SLOPE_STEEP_N:
-		z = 1 + (((x ^ 0xF) + (y ^ 0xF)) >> 1);
-		break;
+		case SLOPE_STEEP_N:
+			z = 1 + (((x ^ 0xF) + (y ^ 0xF)) >> 1);
+			break;
 
-	case SLOPE_STEEP_E:
-		z = 1 + (((x ^ 0xF) + y) >> 1);
-		break;
+		case SLOPE_STEEP_E:
+			z = 1 + (((x ^ 0xF) + y) >> 1);
+			break;
 
 		default: break;
 	}
@@ -299,7 +321,7 @@ void GetSlopeZOnEdge(Slope tileh, DiagDirection edge, int *z1, int *z2)
  * @param z returns the z of the foundation slope. (Can be NULL, if not needed)
  * @return The slope on top of the foundation.
  */
-Slope GetFoundationSlope(TileIndex tile, uint* z)
+Slope GetFoundationSlope(TileIndex tile, uint *z)
 {
 	Slope tileh = GetTileSlope(tile, z);
 	Foundation f = _tile_type_procs[GetTileType(tile)]->get_foundation_proc(tile, tileh);
@@ -342,7 +364,11 @@ static bool HasFoundationNE(TileIndex tile, Slope slope_here, uint z_here)
 	return (z_N_here > z_N) || (z_E_here > z_E);
 }
 
-
+/**
+ * Draw foundation \a f at tile \a ti. Updates \a ti.
+ * @param ti Tile to draw foundation on
+ * @param f  Foundation to draw
+ */
 void DrawFoundation(TileInfo *ti, Foundation f)
 {
 	if (!IsFoundation(f)) return;
@@ -383,7 +409,11 @@ void DrawFoundation(TileInfo *ti, Foundation f)
 			/* inclined foundation */
 			byte inclined = highest_corner * 2 + (f == FOUNDATION_INCLINED_Y ? 1 : 0);
 
-			AddSortableSpriteToDraw(inclined_base + inclined, PAL_NONE, ti->x, ti->y, 16, 16, 1, ti->z);
+			AddSortableSpriteToDraw(inclined_base + inclined, PAL_NONE, ti->x, ti->y,
+				f == FOUNDATION_INCLINED_X ? 16 : 1,
+				f == FOUNDATION_INCLINED_Y ? 16 : 1,
+				TILE_HEIGHT, ti->z
+			);
 			OffsetGroundSprite(31, 9);
 		} else if (IsLeveledFoundation(f)) {
 			AddSortableSpriteToDraw(leveled_base + SlopeWithOneCornerRaised(highest_corner), PAL_NONE, ti->x, ti->y, 16, 16, 7, ti->z - TILE_HEIGHT);
@@ -428,7 +458,11 @@ void DrawFoundation(TileInfo *ti, Foundation f)
 			/* inclined foundation */
 			byte inclined = GetHighestSlopeCorner(ti->tileh) * 2 + (f == FOUNDATION_INCLINED_Y ? 1 : 0);
 
-			AddSortableSpriteToDraw(inclined_base + inclined, PAL_NONE, ti->x, ti->y, 16, 16, 1, ti->z);
+			AddSortableSpriteToDraw(inclined_base + inclined, PAL_NONE, ti->x, ti->y,
+				f == FOUNDATION_INCLINED_X ? 16 : 1,
+				f == FOUNDATION_INCLINED_Y ? 16 : 1,
+				TILE_HEIGHT, ti->z
+			);
 			OffsetGroundSprite(31, 9);
 		}
 		ti->z += ApplyFoundationToSlope(f, &ti->tileh);
@@ -455,9 +489,15 @@ TrackStatus GetTileTrackStatus(TileIndex tile, TransportType mode, uint sub_mode
 	return _tile_type_procs[GetTileType(tile)]->get_tile_track_status_proc(tile, mode, sub_mode, side);
 }
 
-void ChangeTileOwner(TileIndex tile, PlayerID old_player, PlayerID new_player)
+/**
+ * Change the owner of a tile
+ * @param tile      Tile to change
+ * @param old_owner Current owner of the tile
+ * @param new_owner New owner of the tile
+ */
+void ChangeTileOwner(TileIndex tile, Owner old_owner, Owner new_owner)
 {
-	_tile_type_procs[GetTileType(tile)]->change_tile_owner_proc(tile, old_player, new_player);
+	_tile_type_procs[GetTileType(tile)]->change_tile_owner_proc(tile, old_owner, new_owner);
 }
 
 void GetAcceptedCargo(TileIndex tile, AcceptedCargo ac)
@@ -471,9 +511,9 @@ void AnimateTile(TileIndex tile)
 	_tile_type_procs[GetTileType(tile)]->animate_tile_proc(tile);
 }
 
-void ClickTile(TileIndex tile)
+bool ClickTile(TileIndex tile)
 {
-	_tile_type_procs[GetTileType(tile)]->click_tile_proc(tile);
+	return _tile_type_procs[GetTileType(tile)]->click_tile_proc(tile);
 }
 
 void GetTileDesc(TileIndex tile, TileDesc *td)
@@ -484,6 +524,7 @@ void GetTileDesc(TileIndex tile, TileDesc *td)
 /**
  * Has a snow line table already been loaded.
  * @return true if the table has been loaded already.
+ * @ingroup SnowLineGroup
  */
 bool IsSnowLineSet(void)
 {
@@ -493,15 +534,18 @@ bool IsSnowLineSet(void)
 /**
  * Set a variable snow line, as loaded from a newgrf file.
  * @param table the 12 * 32 byte table containing the snowline for each day
+ * @ingroup SnowLineGroup
  */
 void SetSnowLine(byte table[SNOW_LINE_MONTHS][SNOW_LINE_DAYS])
 {
 	_snow_line = CallocT<SnowLine>(1);
+	_snow_line->lowest_value = 0xFF;
 	memcpy(_snow_line->table, table, sizeof(_snow_line->table));
 
 	for (uint i = 0; i < SNOW_LINE_MONTHS; i++) {
 		for (uint j = 0; j < SNOW_LINE_DAYS; j++) {
 			_snow_line->highest_value = max(_snow_line->highest_value, table[i][j]);
+			_snow_line->lowest_value = min(_snow_line->lowest_value, table[i][j]);
 		}
 	}
 }
@@ -509,10 +553,11 @@ void SetSnowLine(byte table[SNOW_LINE_MONTHS][SNOW_LINE_DAYS])
 /**
  * Get the current snow line, either variable or static.
  * @return the snow line height.
+ * @ingroup SnowLineGroup
  */
 byte GetSnowLine(void)
 {
-	if (_snow_line == NULL) return _opt.snow_line;
+	if (_snow_line == NULL) return _settings_game.game_creation.snow_line;
 
 	YearMonthDay ymd;
 	ConvertDateToYMD(_date, &ymd);
@@ -522,14 +567,26 @@ byte GetSnowLine(void)
 /**
  * Get the highest possible snow line height, either variable or static.
  * @return the highest snow line height.
+ * @ingroup SnowLineGroup
  */
 byte HighestSnowLine(void)
 {
-	return _snow_line == NULL ? _opt.snow_line : _snow_line->highest_value;
+	return _snow_line == NULL ? _settings_game.game_creation.snow_line : _snow_line->highest_value;
+}
+
+/**
+ * Get the lowest possible snow line height, either variable or static.
+ * @return the lowest snow line height.
+ * @ingroup SnowLineGroup
+ */
+byte LowestSnowLine(void)
+{
+	return _snow_line == NULL ? _settings_game.game_creation.snow_line : _snow_line->lowest_value;
 }
 
 /**
  * Clear the variable snow line table and free the memory.
+ * @ingroup SnowLineGroup
  */
 void ClearSnowLine(void)
 {
@@ -543,7 +600,7 @@ void ClearSnowLine(void)
  * @param p1 unused
  * @param p2 unused
  */
-CommandCost CmdLandscapeClear(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
+CommandCost CmdLandscapeClear(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
 	return _tile_type_procs[GetTileType(tile)]->clear_tile_proc(tile, flags);
 }
@@ -554,37 +611,31 @@ CommandCost CmdLandscapeClear(TileIndex tile, uint32 flags, uint32 p1, uint32 p2
  * @param flags of operation to conduct
  * @param p2 unused
  */
-CommandCost CmdClearArea(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
+CommandCost CmdClearArea(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	CommandCost ret, money;
-	CommandCost cost(EXPENSES_CONSTRUCTION);
-	int ex;
-	int ey;
-	int sx, sy;
-	int x, y;
-	bool success = false;
-
 	if (p1 >= MapSize()) return CMD_ERROR;
 
 	/* make sure sx,sy are smaller than ex,ey */
-	ex = TileX(tile);
-	ey = TileY(tile);
-	sx = TileX(p1);
-	sy = TileY(p1);
+	int ex = TileX(tile);
+	int ey = TileY(tile);
+	int sx = TileX(p1);
+	int sy = TileY(p1);
 	if (ex < sx) Swap(ex, sx);
 	if (ey < sy) Swap(ey, sy);
 
-	money.AddCost(GetAvailableMoneyForCommand());
+	Money money = GetAvailableMoneyForCommand();
+	CommandCost cost(EXPENSES_CONSTRUCTION);
+	bool success = false;
 
-	for (x = sx; x <= ex; ++x) {
-		for (y = sy; y <= ey; ++y) {
-			ret = DoCommand(TileXY(x, y), 0, 0, flags & ~DC_EXEC, CMD_LANDSCAPE_CLEAR);
+	for (int x = sx; x <= ex; ++x) {
+		for (int y = sy; y <= ey; ++y) {
+			CommandCost ret = DoCommand(TileXY(x, y), 0, 0, flags & ~DC_EXEC, CMD_LANDSCAPE_CLEAR);
 			if (CmdFailed(ret)) continue;
 			success = true;
 
 			if (flags & DC_EXEC) {
-				money.AddCost(-ret.GetCost());
-				if (ret.GetCost() > 0 && money.GetCost() < 0) {
+				money -= ret.GetCost();
+				if (ret.GetCost() > 0 && money < 0) {
 					_additional_cash_required = ret.GetCost();
 					return cost;
 				}
@@ -606,6 +657,7 @@ CommandCost CmdClearArea(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
 }
 
 
+TileIndex _cur_tileloop_tile;
 #define TILELOOP_BITS 4
 #define TILELOOP_SIZE (1 << TILELOOP_BITS)
 #define TILELOOP_ASSERTMASK ((TILELOOP_SIZE - 1) + ((TILELOOP_SIZE - 1) << MapLogX()))
@@ -613,27 +665,25 @@ CommandCost CmdClearArea(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
 
 void RunTileLoop()
 {
-	TileIndex tile;
-	uint count;
+	TileIndex tile = _cur_tileloop_tile;
 
-	tile = _cur_tileloop_tile;
-
-	assert( (tile & ~TILELOOP_ASSERTMASK) == 0);
-	count = (MapSizeX() / TILELOOP_SIZE) * (MapSizeY() / TILELOOP_SIZE);
+	assert((tile & ~TILELOOP_ASSERTMASK) == 0);
+	uint count = (MapSizeX() / TILELOOP_SIZE) * (MapSizeY() / TILELOOP_SIZE);
 	do {
 		_tile_type_procs[GetTileType(tile)]->tile_loop_proc(tile);
 
 		if (TileX(tile) < MapSizeX() - TILELOOP_SIZE) {
 			tile += TILELOOP_SIZE; // no overflow
 		} else {
-			tile = TILE_MASK(tile - TILELOOP_SIZE * (MapSizeX() / TILELOOP_SIZE - 1) + TileDiffXY(0, TILELOOP_SIZE)); /* x would overflow, also increase y */
+			tile = TILE_MASK(tile - TILELOOP_SIZE * (MapSizeX() / TILELOOP_SIZE - 1) + TileDiffXY(0, TILELOOP_SIZE)); // x would overflow, also increase y
 		}
-	} while (--count);
-	assert( (tile & ~TILELOOP_ASSERTMASK) == 0);
+	} while (--count != 0);
+	assert((tile & ~TILELOOP_ASSERTMASK) == 0);
 
 	tile += 9;
-	if (tile & TILELOOP_CHKMASK)
+	if (tile & TILELOOP_CHKMASK) {
 		tile = (tile + MapSizeX()) & TILELOOP_ASSERTMASK;
+	}
 	_cur_tileloop_tile = tile;
 }
 
@@ -642,11 +692,11 @@ void InitializeLandscape()
 	uint maxx = MapMaxX();
 	uint maxy = MapMaxY();
 	uint sizex = MapSizeX();
-	uint x;
-	uint y;
 
-	for (y = 0; y < maxy; y++) {
-		for (x = 0; x < maxx; x++) {
+	uint y;
+	for (y = _settings_game.construction.freeform_edges ? 1 : 0; y < maxy; y++) {
+		uint x;
+		for (x = _settings_game.construction.freeform_edges ? 1 : 0; x < maxx; x++) {
 			MakeClear(sizex * y + x, CLEAR_GRASS, 3);
 			SetTileHeight(sizex * y + x, 0);
 			SetTropicZone(sizex * y + x, TROPICZONE_NORMAL);
@@ -654,49 +704,38 @@ void InitializeLandscape()
 		}
 		MakeVoid(sizex * y + x);
 	}
-	for (x = 0; x < sizex; x++) MakeVoid(sizex * y + x);
+	for (uint x = 0; x < sizex; x++) MakeVoid(sizex * y + x);
 }
 
 static const byte _genterrain_tbl_1[5] = { 10, 22, 33, 37, 4  };
 static const byte _genterrain_tbl_2[5] = {  0,  0,  0,  0, 33 };
 
-static void GenerateTerrain(int type, int flag)
+static void GenerateTerrain(int type, uint flag)
 {
-	uint32 r;
-	uint x;
-	uint y;
-	uint w;
-	uint h;
-	const Sprite* templ;
-	const byte *p;
-	Tile* tile;
-	byte direction;
+	uint32 r = Random();
 
-	r = Random();
-	templ = GetSprite((((r >> 24) * _genterrain_tbl_1[type]) >> 8) + _genterrain_tbl_2[type] + 4845);
-
-	x = r & MapMaxX();
-	y = (r >> MapLogX()) & MapMaxY();
+	const Sprite *templ = GetSprite((((r >> 24) * _genterrain_tbl_1[type]) >> 8) + _genterrain_tbl_2[type] + 4845, ST_MAPGEN);
 
+	uint x = r & MapMaxX();
+	uint y = (r >> MapLogX()) & MapMaxY();
 
 	if (x < 2 || y < 2) return;
 
-	direction = GB(r, 22, 2);
-	if (direction & 1) {
-		w = templ->height;
-		h = templ->width;
-	} else {
-		w = templ->width;
-		h = templ->height;
-	}
-	p = templ->data;
+	DiagDirection direction = (DiagDirection)GB(r, 22, 2);
+	uint w = templ->width;
+	uint h = templ->height;
 
-	if (flag & 4) {
+	if (DiagDirToAxis(direction) == AXIS_Y) Swap(w, h);
+
+	const byte *p = templ->data;
+
+	if ((flag & 4) != 0) {
 		uint xw = x * MapSizeY();
 		uint yw = y * MapSizeX();
 		uint bias = (MapSizeX() + MapSizeY()) * 16;
 
 		switch (flag & 3) {
+			default: NOT_REACHED();
 			case 0:
 				if (xw + yw > MapSize() - bias) return;
 				break;
@@ -718,16 +757,16 @@ static void GenerateTerrain(int type, int flag)
 	if (x + w >= MapMaxX() - 1) return;
 	if (y + h >= MapMaxY() - 1) return;
 
-	tile = &_m[TileXY(x, y)];
+	Tile *tile = &_m[TileXY(x, y)];
 
 	switch (direction) {
-		case 0:
+		default: NOT_REACHED();
+		case DIAGDIR_NE:
 			do {
-				Tile* tile_cur = tile;
-				uint w_cur;
+				Tile *tile_cur = tile;
 
-				for (w_cur = w; w_cur != 0; --w_cur) {
-					if (*p >= tile_cur->type_height) tile_cur->type_height = *p;
+				for (uint w_cur = w; w_cur != 0; --w_cur) {
+					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
 					p++;
 					tile_cur++;
 				}
@@ -735,28 +774,26 @@ static void GenerateTerrain(int type, int flag)
 			} while (--h != 0);
 			break;
 
-		case 1:
+		case DIAGDIR_SE:
 			do {
-				Tile* tile_cur = tile;
-				uint h_cur;
+				Tile *tile_cur = tile;
 
-				for (h_cur = h; h_cur != 0; --h_cur) {
-					if (*p >= tile_cur->type_height) tile_cur->type_height = *p;
+				for (uint h_cur = h; h_cur != 0; --h_cur) {
+					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
 					p++;
 					tile_cur += TileDiffXY(0, 1);
 				}
-				tile++;
+				tile += TileDiffXY(1, 0);
 			} while (--w != 0);
 			break;
 
-		case 2:
+		case DIAGDIR_SW:
 			tile += TileDiffXY(w - 1, 0);
 			do {
-				Tile* tile_cur = tile;
-				uint w_cur;
+				Tile *tile_cur = tile;
 
-				for (w_cur = w; w_cur != 0; --w_cur) {
-					if (*p >= tile_cur->type_height) tile_cur->type_height = *p;
+				for (uint w_cur = w; w_cur != 0; --w_cur) {
+					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
 					p++;
 					tile_cur--;
 				}
@@ -764,18 +801,17 @@ static void GenerateTerrain(int type, int flag)
 			} while (--h != 0);
 			break;
 
-		case 3:
+		case DIAGDIR_NW:
 			tile += TileDiffXY(0, h - 1);
 			do {
-				Tile* tile_cur = tile;
-				uint h_cur;
+				Tile *tile_cur = tile;
 
-				for (h_cur = h; h_cur != 0; --h_cur) {
-					if (*p >= tile_cur->type_height) tile_cur->type_height = *p;
+				for (uint h_cur = h; h_cur != 0; --h_cur) {
+					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
 					p++;
 					tile_cur -= TileDiffXY(0, 1);
 				}
-				tile++;
+				tile += TileDiffXY(1, 0);
 			} while (--w != 0);
 			break;
 	}
@@ -786,36 +822,34 @@ static void GenerateTerrain(int type, int flag)
 
 static void CreateDesertOrRainForest()
 {
-	TileIndex tile;
 	TileIndex update_freq = MapSize() / 4;
 	const TileIndexDiffC *data;
-	uint i;
 
-	for (tile = 0; tile != MapSize(); ++tile) {
+	for (TileIndex tile = 0; tile != MapSize(); ++tile) {
 		if ((tile % update_freq) == 0) IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 		for (data = _make_desert_or_rainforest_data;
 				data != endof(_make_desert_or_rainforest_data); ++data) {
-			TileIndex t = TILE_MASK(tile + ToTileIndexDiff(*data));
-			if (TileHeight(t) >= 4 || IsTileType(t, MP_WATER)) break;
+			TileIndex t = AddTileIndexDiffCWrap(tile, *data);
+			if (t != INVALID_TILE && (TileHeight(t) >= 4 || IsTileType(t, MP_WATER))) break;
 		}
 		if (data == endof(_make_desert_or_rainforest_data))
 			SetTropicZone(tile, TROPICZONE_DESERT);
 	}
 
-	for (i = 0; i != 256; i++) {
+	for (uint i = 0; i != 256; i++) {
 		if ((i % 64) == 0) IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 		RunTileLoop();
 	}
 
-	for (tile = 0; tile != MapSize(); ++tile) {
+	for (TileIndex tile = 0; tile != MapSize(); ++tile) {
 		if ((tile % update_freq) == 0) IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 		for (data = _make_desert_or_rainforest_data;
 				data != endof(_make_desert_or_rainforest_data); ++data) {
-			TileIndex t = TILE_MASK(tile + ToTileIndexDiff(*data));
-			if (IsTileType(t, MP_CLEAR) && IsClearGround(t, CLEAR_DESERT)) break;
+			TileIndex t = AddTileIndexDiffCWrap(tile, *data);
+			if (t != INVALID_TILE && IsTileType(t, MP_CLEAR) && IsClearGround(t, CLEAR_DESERT)) break;
 		}
 		if (data == endof(_make_desert_or_rainforest_data))
 			SetTropicZone(tile, TROPICZONE_RAINFOREST);
@@ -824,74 +858,80 @@ static void CreateDesertOrRainForest()
 
 void GenerateLandscape(byte mode)
 {
-	const int gwp_desert_amount = 4 + 8;
-	uint i;
-	uint flag;
-	uint32 r;
+	static const int gwp_desert_amount = 4 + 8;
 
 	if (mode == GW_HEIGHTMAP) {
-		SetGeneratingWorldProgress(GWP_LANDSCAPE, (_opt.landscape == LT_TROPIC) ? 1 + gwp_desert_amount : 1);
+		SetGeneratingWorldProgress(GWP_LANDSCAPE, (_settings_game.game_creation.landscape == LT_TROPIC) ? 1 + gwp_desert_amount : 1);
 		LoadHeightmap(_file_to_saveload.name);
 		IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
-	} else if (_patches.land_generator == LG_TERRAGENESIS) {
-		SetGeneratingWorldProgress(GWP_LANDSCAPE, (_opt.landscape == LT_TROPIC) ? 3 + gwp_desert_amount : 3);
+	} else if (_settings_game.game_creation.land_generator == LG_TERRAGENESIS) {
+		SetGeneratingWorldProgress(GWP_LANDSCAPE, (_settings_game.game_creation.landscape == LT_TROPIC) ? 3 + gwp_desert_amount : 3);
 		GenerateTerrainPerlin();
 	} else {
-		switch (_opt.landscape) {
-			case LT_ARCTIC:
+		if (_settings_game.construction.freeform_edges) {
+			for (uint x = 0; x < MapSizeX(); x++) MakeVoid(TileXY(x, 0));
+			for (uint y = 0; y < MapSizeY(); y++) MakeVoid(TileXY(0, y));
+		}
+		switch (_settings_game.game_creation.landscape) {
+			case LT_ARCTIC: {
 				SetGeneratingWorldProgress(GWP_LANDSCAPE, 2);
 
-				for (i = ScaleByMapSize((Random() & 0x7F) + 950); i != 0; --i) {
+				uint32 r = Random();
+
+				for (uint i = ScaleByMapSize(GB(r, 0, 7) + 950); i != 0; --i) {
 					GenerateTerrain(2, 0);
 				}
 				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
-				r = Random();
-				flag = GB(r, 0, 2) | 4;
-				for (i = ScaleByMapSize(GB(r, 16, 7) + 450); i != 0; --i) {
+				uint flag = GB(r, 7, 2) | 4;
+				for (uint i = ScaleByMapSize(GB(r, 9, 7) + 450); i != 0; --i) {
 					GenerateTerrain(4, flag);
 				}
 				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
-				break;
+			} break;
 
-			case LT_TROPIC:
+			case LT_TROPIC: {
 				SetGeneratingWorldProgress(GWP_LANDSCAPE, 3 + gwp_desert_amount);
 
-				for (i = ScaleByMapSize((Random() & 0x7F) + 170); i != 0; --i) {
+				uint32 r = Random();
+
+				for (uint i = ScaleByMapSize(GB(r, 0, 7) + 170); i != 0; --i) {
 					GenerateTerrain(0, 0);
 				}
 				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
-				r = Random();
-				flag = GB(r, 0, 2) | 4;
-				for (i = ScaleByMapSize(GB(r, 16, 8) + 1700); i != 0; --i) {
+				uint flag = GB(r, 7, 2) | 4;
+				for (uint i = ScaleByMapSize(GB(r, 9, 8) + 1700); i != 0; --i) {
 					GenerateTerrain(0, flag);
 				}
 				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 				flag ^= 2;
 
-				for (i = ScaleByMapSize((Random() & 0x7F) + 410); i != 0; --i) {
+				for (uint i = ScaleByMapSize(GB(r, 17, 7) + 410); i != 0; --i) {
 					GenerateTerrain(3, flag);
 				}
 				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
-				break;
+			} break;
 
-			default:
+			default: {
 				SetGeneratingWorldProgress(GWP_LANDSCAPE, 1);
 
-				i = ScaleByMapSize((Random() & 0x7F) + (3 - _opt.diff.quantity_sea_lakes) * 256 + 100);
+				uint32 r = Random();
+
+				uint i = ScaleByMapSize(GB(r, 0, 7) + (3 - _settings_game.difficulty.quantity_sea_lakes) * 256 + 100);
 				for (; i != 0; --i) {
-					GenerateTerrain(_opt.diff.terrain_type, 0);
+					GenerateTerrain(_settings_game.difficulty.terrain_type, 0);
 				}
 				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
-				break;
+			} break;
 		}
 	}
 
+	FixSlopes();
 	ConvertGroundTilesIntoWaterTiles();
 
-	if (_opt.landscape == LT_TROPIC) CreateDesertOrRainForest();
+	if (_settings_game.game_creation.landscape == LT_TROPIC) CreateDesertOrRainForest();
 }
 
 void OnTick_Town();
@@ -899,7 +939,7 @@ void OnTick_Trees();
 void OnTick_Station();
 void OnTick_Industry();
 
-void OnTick_Players();
+void OnTick_Companies();
 void OnTick_Train();
 
 void CallLandscapeTick()
@@ -909,17 +949,6 @@ void CallLandscapeTick()
 	OnTick_Station();
 	OnTick_Industry();
 
-	OnTick_Players();
+	OnTick_Companies();
 	OnTick_Train();
 }
-
-TileIndex AdjustTileCoordRandomly(TileIndex a, byte rng)
-{
-	int rn = rng;
-	uint32 r = Random();
-
-	return TILE_MASK(TileXY(
-		TileX(a) + (GB(r, 0, 8) * rn * 2 >> 8) - rn,
-		TileY(a) + (GB(r, 8, 8) * rn * 2 >> 8) - rn
-	));
-}
