@@ -1,4 +1,4 @@
-/* $Id: network_udp.cpp 13735 2008-07-19 12:23:14Z rubidium $ */
+/* $Id: network_udp.cpp 16060 2009-04-14 21:13:07Z rubidium $ */
 
 /**
  * @file network_udp.cpp This file handles the UDP related communication.
@@ -10,31 +10,33 @@
 #ifdef ENABLE_NETWORK
 
 #include "../stdafx.h"
-#include "../debug.h"
-#include "network_data.h"
 #include "../date_func.h"
 #include "../map_func.h"
 #include "network_gamelist.h"
+#include "network_internal.h"
 #include "network_udp.h"
-#include "../variables.h"
-#include "../newgrf_config.h"
+#include "core/host.h"
 #include "../core/endian_func.hpp"
+#include "../core/alloc_func.hpp"
 #include "../string_func.h"
-#include "../player_base.h"
-#include "../player_func.h"
+#include "../company_base.h"
 #include "../settings_type.h"
+#include "../thread.h"
+#include "../rev.h"
 
 #include "core/udp.h"
 
+ThreadMutex *_network_udp_mutex = ThreadMutex::New();
+
 enum {
 	ADVERTISE_NORMAL_INTERVAL = 30000, // interval between advertising in ticks (15 minutes)
 	ADVERTISE_RETRY_INTERVAL  =   300, // readvertise when no response after this many ticks (9 seconds)
 	ADVERTISE_RETRY_TIMES     =     3  // give up readvertising after this much failed retries
 };
 
-NetworkUDPSocketHandler *_udp_client_socket; ///< udp client socket
-NetworkUDPSocketHandler *_udp_server_socket; ///< udp server socket
-NetworkUDPSocketHandler *_udp_master_socket; ///< udp master socket
+NetworkUDPSocketHandler *_udp_client_socket = NULL; ///< udp client socket
+NetworkUDPSocketHandler *_udp_server_socket = NULL; ///< udp server socket
+NetworkUDPSocketHandler *_udp_master_socket = NULL; ///< udp master socket
 
 ///*** Communication with the masterserver ***/
 
@@ -51,7 +53,7 @@ DEF_UDP_RECEIVE_COMMAND(Master, PACKET_UDP_MASTER_ACK_REGISTER)
 	DEBUG(net, 2, "[udp] advertising on master server successful");
 
 	/* We are advertised, but we don't want to! */
-	if (!_network_advertise) NetworkUDPRemoveAdvertise();
+	if (!_settings_client.network.server_advertise) NetworkUDPRemoveAdvertise(false);
 }
 
 ///*** Communication with clients (we are server) ***/
@@ -67,24 +69,39 @@ public:
 
 DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_FIND_SERVER)
 {
-	// Just a fail-safe.. should never happen
-	if (!_network_udp_server)
+	/* Just a fail-safe.. should never happen */
+	if (!_network_udp_server) {
 		return;
+	}
+
+	NetworkGameInfo ngi;
+
+	/* Update some game_info */
+	ngi.clients_on     = _network_game_info.clients_on;
+	ngi.start_date     = _network_game_info.start_date;
+
+	ngi.server_lang    = _settings_client.network.server_lang;
+	ngi.use_password   = !StrEmpty(_settings_client.network.server_password);
+	ngi.clients_max    = _settings_client.network.max_clients;
+	ngi.companies_on   = ActiveCompanyCount();
+	ngi.companies_max  = _settings_client.network.max_companies;
+	ngi.spectators_on  = NetworkSpectatorCount();
+	ngi.spectators_max = _settings_client.network.max_spectators;
+	ngi.game_date      = _date;
+	ngi.map_width      = MapSizeX();
+	ngi.map_height     = MapSizeY();
+	ngi.map_set        = _settings_game.game_creation.landscape;
+	ngi.dedicated      = _network_dedicated;
+	ngi.grfconfig      = _grfconfig;
+
+	strecpy(ngi.map_name, _network_game_info.map_name, lastof(ngi.map_name));
+	strecpy(ngi.server_name, _settings_client.network.server_name, lastof(ngi.server_name));
+	strecpy(ngi.server_revision, _openttd_revision, lastof(ngi.server_revision));
 
 	Packet packet(PACKET_UDP_SERVER_RESPONSE);
+	this->Send_NetworkGameInfo(&packet, &ngi);
 
-	// Update some game_info
-	_network_game_info.game_date     = _date;
-	_network_game_info.map_width     = MapSizeX();
-	_network_game_info.map_height    = MapSizeY();
-	_network_game_info.map_set       = _opt.landscape;
-	_network_game_info.companies_on  = ActivePlayerCount();
-	_network_game_info.spectators_on = NetworkSpectatorCount();
-	_network_game_info.grfconfig     = _grfconfig;
-
-	this->Send_NetworkGameInfo(&packet, &_network_game_info);
-
-	// Let the client know that we are here
+	/* Let the client know that we are here */
 	this->SendPacket(&packet, client_addr);
 
 	DEBUG(net, 2, "[udp] queried from '%s'", inet_ntoa(client_addr->sin_addr));
@@ -92,47 +109,24 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_FIND_SERVER)
 
 DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_DETAIL_INFO)
 {
-	// Just a fail-safe.. should never happen
+	/* Just a fail-safe.. should never happen */
 	if (!_network_udp_server) return;
 
 	Packet packet(PACKET_UDP_SERVER_DETAIL_INFO);
 
 	/* Send the amount of active companies */
 	packet.Send_uint8 (NETWORK_COMPANY_INFO_VERSION);
-	packet.Send_uint8 (ActivePlayerCount());
+	packet.Send_uint8 (ActiveCompanyCount());
 
-	/* Fetch the latest version of everything */
-	NetworkPopulateCompanyInfo();
-
-	Player *player;
-	byte current = 0;
-	/* Go through all the players */
-	FOR_ALL_PLAYERS(player) {
-		/* Skip non-active players */
-		if (!player->is_active) continue;
-
-		current++;
+	/* Fetch the latest version of the stats */
+	NetworkCompanyStats company_stats[MAX_COMPANIES];
+	NetworkPopulateCompanyStats(company_stats);
 
+	Company *company;
+	/* Go through all the companies */
+	FOR_ALL_COMPANIES(company) {
 		/* Send the information */
-		packet.Send_uint8 (current);
-
-		packet.Send_string(_network_player_info[player->index].company_name);
-		packet.Send_uint32(_network_player_info[player->index].inaugurated_year);
-		packet.Send_uint64(_network_player_info[player->index].company_value);
-		packet.Send_uint64(_network_player_info[player->index].money);
-		packet.Send_uint64(_network_player_info[player->index].income);
-		packet.Send_uint16(_network_player_info[player->index].performance);
-
-		/* Send 1 if there is a passord for the company else send 0 */
-		packet.Send_bool  (!StrEmpty(_network_player_info[player->index].password));
-
-		for (int i = 0; i < NETWORK_VEHICLE_TYPES; i++) {
-			packet.Send_uint16(_network_player_info[player->index].num_vehicle[i]);
-		}
-
-		for (int i = 0; i < NETWORK_STATION_TYPES; i++) {
-			packet.Send_uint16(_network_player_info[player->index].num_station[i]);
-		}
+		this->Send_CompanyInformation(&packet, company, &company_stats[company->index]);
 	}
 
 	this->SendPacket(&packet, client_addr);
@@ -158,7 +152,7 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 
 	const GRFConfig *in_reply[NETWORK_MAX_GRF_COUNT];
 	uint8 in_reply_count = 0;
-	uint packet_len = 0;
+	size_t packet_len = 0;
 
 	DEBUG(net, 6, "[udp] newgrf data request from %s:%d", inet_ntoa(client_addr->sin_addr), ntohs(client_addr->sin_port));
 
@@ -195,8 +189,8 @@ DEF_UDP_RECEIVE_COMMAND(Server, PACKET_UDP_CLIENT_GET_NEWGRFS)
 		char name[NETWORK_GRF_NAME_LENGTH];
 
 		/* The name could be an empty string, if so take the filename */
-		ttd_strlcpy(name, (in_reply[i]->name != NULL && !StrEmpty(in_reply[i]->name)) ?
-				in_reply[i]->name : in_reply[i]->filename, sizeof(name));
+		strecpy(name, (in_reply[i]->name != NULL && !StrEmpty(in_reply[i]->name)) ?
+				in_reply[i]->name : in_reply[i]->filename, lastof(name));
 		this->Send_GRFIdentifier(&packet, in_reply[i]);
 		packet.Send_string(name);
 	}
@@ -220,12 +214,12 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 {
 	NetworkGameList *item;
 
-	// Just a fail-safe.. should never happen
+	/* Just a fail-safe.. should never happen */
 	if (_network_udp_server) return;
 
 	DEBUG(net, 4, "[udp] server response from %s:%d", inet_ntoa(client_addr->sin_addr), ntohs(client_addr->sin_port));
 
-	// Find next item
+	/* Find next item */
 	item = NetworkGameListAddItem(inet_addr(inet_ntoa(client_addr->sin_addr)), ntohs(client_addr->sin_port));
 
 	this->Recv_NetworkGameInfo(p, &item->info);
@@ -246,7 +240,7 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 
 		for (c = item->info.grfconfig; c != NULL; c = c->next) {
 			if (c->status == GCS_NOT_FOUND) item->info.compatible = false;
- 			if (c->status != GCS_NOT_FOUND || strcmp(c->name, UNKNOWN_GRF_NAME_PLACEHOLDER) != 0) continue;
+			if (c->status != GCS_NOT_FOUND || strcmp(c->name, UNKNOWN_GRF_NAME_PLACEHOLDER) != 0) continue;
 			in_request[in_request_count] = c;
 			in_request_count++;
 		}
@@ -282,27 +276,22 @@ DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_SERVER_RESPONSE)
 
 DEF_UDP_RECEIVE_COMMAND(Client, PACKET_UDP_MASTER_RESPONSE_LIST)
 {
-	int i;
-	struct in_addr ip;
-	uint16 port;
-	uint8 ver;
-
 	/* packet begins with the protocol version (uint8)
 	 * then an uint16 which indicates how many
 	 * ip:port pairs are in this packet, after that
 	 * an uint32 (ip) and an uint16 (port) for each pair
 	 */
 
-	ver = p->Recv_uint8();
+	uint8 ver = p->Recv_uint8();
 
 	if (ver == 1) {
-		for (i = p->Recv_uint16(); i != 0 ; i--) {
-			ip.s_addr = TO_LE32(p->Recv_uint32());
-			port = p->Recv_uint16();
+		for (int i = p->Recv_uint16(); i != 0 ; i--) {
+			uint32 ip = TO_LE32(p->Recv_uint32());
+			uint16 port = p->Recv_uint16();
 
 			/* Somehow we reached the end of the packet */
 			if (this->HasClientQuit()) return;
-			NetworkUDPQueryServer(inet_ntoa(ip), port);
+			NetworkUDPQueryServer(NetworkAddress(ip, port));
 		}
 	}
 }
@@ -358,20 +347,22 @@ void ClientNetworkUDPSocketHandler::HandleIncomingNetworkGameInfoGRFConfig(GRFCo
 	SetBit(config->flags, GCF_COPY);
 }
 
-// Close UDP connection
+/* Close UDP connection */
 void NetworkUDPCloseAll()
 {
 	DEBUG(net, 1, "[udp] closed listeners");
 
+	_network_udp_mutex->BeginCritical();
 	_udp_server_socket->Close();
 	_udp_master_socket->Close();
 	_udp_client_socket->Close();
+	_network_udp_mutex->EndCritical();
 
 	_network_udp_server = false;
 	_network_udp_broadcast = 0;
 }
 
-// Broadcast to all ips
+/* Broadcast to all ips */
 static void NetworkUDPBroadCast(NetworkUDPSocketHandler *socket)
 {
 	uint i;
@@ -381,7 +372,7 @@ static void NetworkUDPBroadCast(NetworkUDPSocketHandler *socket)
 		struct sockaddr_in out_addr;
 
 		out_addr.sin_family = AF_INET;
-		out_addr.sin_port = htons(_network_server_port);
+		out_addr.sin_port = htons(_settings_client.network.server_port);
 		out_addr.sin_addr.s_addr = _broadcast_list[i];
 
 		DEBUG(net, 4, "[udp] broadcasting to %s", inet_ntoa(out_addr.sin_addr));
@@ -391,7 +382,7 @@ static void NetworkUDPBroadCast(NetworkUDPSocketHandler *socket)
 }
 
 
-// Request the the server-list from the master server
+/* Request the the server-list from the master server */
 void NetworkUDPQueryMasterServer()
 {
 	struct sockaddr_in out_addr;
@@ -406,7 +397,7 @@ void NetworkUDPQueryMasterServer()
 	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
 	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
 
-	// packet only contains protocol version
+	/* packet only contains protocol version */
 	p.Send_uint8(NETWORK_MASTER_SERVER_VERSION);
 
 	_udp_client_socket->SendPacket(&p, &out_addr);
@@ -414,13 +405,13 @@ void NetworkUDPQueryMasterServer()
 	DEBUG(net, 2, "[udp] master server queried at %s:%d", inet_ntoa(out_addr.sin_addr), ntohs(out_addr.sin_port));
 }
 
-// Find all servers
+/* Find all servers */
 void NetworkUDPSearchGame()
 {
-	// We are still searching..
+	/* We are still searching.. */
 	if (_network_udp_broadcast > 0) return;
 
-	// No UDP-socket yet..
+	/* No UDP-socket yet.. */
 	if (!_udp_client_socket->IsConnected()) {
 		if (!_udp_client_socket->Listen(0, 0, true)) return;
 	}
@@ -431,53 +422,66 @@ void NetworkUDPSearchGame()
 	_network_udp_broadcast = 300; // Stay searching for 300 ticks
 }
 
-void NetworkUDPQueryServer(const char* host, unsigned short port, bool manually)
-{
-	struct sockaddr_in out_addr;
-	NetworkGameList *item;
+/** Simpler wrapper struct for NetworkUDPQueryServerThread */
+struct NetworkUDPQueryServerInfo : NetworkAddress {
+	bool manually; ///< Did we connect manually or not?
+	NetworkUDPQueryServerInfo(const NetworkAddress &address, bool manually) :
+		NetworkAddress(address),
+		manually(manually)
+	{
+	}
+};
 
-	// No UDP-socket yet..
+/**
+ * Threaded part for resolving the IP of a server and querying it.
+ * @param pntr the NetworkUDPQueryServerInfo.
+ */
+void NetworkUDPQueryServerThread(void *pntr)
+{
+	NetworkUDPQueryServerInfo *info = (NetworkUDPQueryServerInfo*)pntr;
+
+	struct sockaddr_in out_addr;
+	out_addr.sin_family = AF_INET;
+	out_addr.sin_port = htons(info->GetPort());
+	out_addr.sin_addr.s_addr = info->GetIP();
+
+	/* Clear item in gamelist */
+	NetworkGameList *item = CallocT<NetworkGameList>(1);
+	item->ip = info->GetIP();
+	item->port = info->GetPort();
+	strecpy(item->info.server_name, info->GetHostname(), lastof(item->info.server_name));
+	strecpy(item->info.hostname, info->GetHostname(), lastof(item->info.hostname));
+	item->manually = info->manually;
+	NetworkGameListAddItemDelayed(item);
+
+	_network_udp_mutex->BeginCritical();
+	/* Init the packet */
+	Packet p(PACKET_UDP_CLIENT_FIND_SERVER);
+	if (_udp_client_socket != NULL) _udp_client_socket->SendPacket(&p, &out_addr);
+	_network_udp_mutex->EndCritical();
+
+	delete info;
+}
+
+void NetworkUDPQueryServer(NetworkAddress address, bool manually)
+{
+	/* No UDP-socket yet.. */
 	if (!_udp_client_socket->IsConnected()) {
 		if (!_udp_client_socket->Listen(0, 0, true)) return;
 	}
 
-	out_addr.sin_family = AF_INET;
-	out_addr.sin_port = htons(port);
-	out_addr.sin_addr.s_addr = NetworkResolveHost(host);
-
-	// Clear item in gamelist
-	item = NetworkGameListAddItem(inet_addr(inet_ntoa(out_addr.sin_addr)), ntohs(out_addr.sin_port));
-	if (StrEmpty(item->info.server_name)) {
-		memset(&item->info, 0, sizeof(item->info));
-		ttd_strlcpy(item->info.server_name, host, lengthof(item->info.server_name));
-		ttd_strlcpy(item->info.hostname, host, lengthof(item->info.hostname));
-		item->online = false;
+	NetworkUDPQueryServerInfo *info = new NetworkUDPQueryServerInfo(address, manually);
+	if (address.IsResolved() || !ThreadObject::New(NetworkUDPQueryServerThread, info)) {
+		NetworkUDPQueryServerThread(info);
 	}
-	item->manually = manually;
-
-	// Init the packet
-	Packet p(PACKET_UDP_CLIENT_FIND_SERVER);
-	_udp_client_socket->SendPacket(&p, &out_addr);
-
-	UpdateNetworkGameWindow(false);
 }
 
-/* Remove our advertise from the master-server */
-void NetworkUDPRemoveAdvertise()
+void NetworkUDPRemoveAdvertiseThread(void *pntr)
 {
-	struct sockaddr_in out_addr;
-
-	/* Check if we are advertising */
-	if (!_networking || !_network_server || !_network_udp_server) return;
-
-	/* check for socket */
-	if (!_udp_master_socket->IsConnected()) {
-		if (!_udp_master_socket->Listen(_network_server_bind_ip, 0, false)) return;
-	}
-
 	DEBUG(net, 1, "[udp] removing advertise from master server");
 
 	/* Find somewhere to send */
+	struct sockaddr_in out_addr;
 	out_addr.sin_family = AF_INET;
 	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
 	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
@@ -486,18 +490,60 @@ void NetworkUDPRemoveAdvertise()
 	Packet p(PACKET_UDP_SERVER_UNREGISTER);
 	/* Packet is: Version, server_port */
 	p.Send_uint8 (NETWORK_MASTER_SERVER_VERSION);
-	p.Send_uint16(_network_server_port);
-	_udp_master_socket->SendPacket(&p, &out_addr);
+	p.Send_uint16(_settings_client.network.server_port);
+
+	_network_udp_mutex->BeginCritical();
+	if (_udp_master_socket != NULL) _udp_master_socket->SendPacket(&p, &out_addr);
+	_network_udp_mutex->EndCritical();
+}
+
+/**
+ * Remove our advertise from the master-server.
+ * @param blocking whether to wait until the removal has finished.
+ */
+void NetworkUDPRemoveAdvertise(bool blocking)
+{
+	/* Check if we are advertising */
+	if (!_networking || !_network_server || !_network_udp_server) return;
+
+	/* check for socket */
+	if (!_udp_master_socket->IsConnected()) {
+		if (!_udp_master_socket->Listen(_network_server_bind_ip, 0, false)) return;
+	}
+
+	if (blocking || !ThreadObject::New(NetworkUDPRemoveAdvertiseThread, NULL)) {
+		NetworkUDPRemoveAdvertiseThread(NULL);
+	}
+}
+
+void NetworkUDPAdvertiseThread(void *pntr)
+{
+	/* Find somewhere to send */
+	struct sockaddr_in out_addr;
+	out_addr.sin_family = AF_INET;
+	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
+	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
+
+	DEBUG(net, 1, "[udp] advertising to master server");
+
+	/* Send the packet */
+	Packet p(PACKET_UDP_SERVER_REGISTER);
+	/* Packet is: WELCOME_MESSAGE, Version, server_port */
+	p.Send_string(NETWORK_MASTER_SERVER_WELCOME_MESSAGE);
+	p.Send_uint8 (NETWORK_MASTER_SERVER_VERSION);
+	p.Send_uint16(_settings_client.network.server_port);
+
+	_network_udp_mutex->BeginCritical();
+	if (_udp_master_socket != NULL) _udp_master_socket->SendPacket(&p, &out_addr);
+	_network_udp_mutex->EndCritical();
 }
 
 /* Register us to the master server
      This function checks if it needs to send an advertise */
 void NetworkUDPAdvertise()
 {
-	struct sockaddr_in out_addr;
-
 	/* Check if we should send an advertise */
-	if (!_networking || !_network_server || !_network_udp_server || !_network_advertise)
+	if (!_networking || !_network_server || !_network_udp_server || !_settings_client.network.server_advertise)
 		return;
 
 	/* check for socket */
@@ -523,39 +569,37 @@ void NetworkUDPAdvertise()
 	_network_advertise_retries--;
 	_network_last_advertise_frame = _frame_counter;
 
-	/* Find somewhere to send */
-	out_addr.sin_family = AF_INET;
-	out_addr.sin_port = htons(NETWORK_MASTER_SERVER_PORT);
-	out_addr.sin_addr.s_addr = NetworkResolveHost(NETWORK_MASTER_SERVER_HOST);
-
-	DEBUG(net, 1, "[udp] advertising to master server");
-
-	/* Send the packet */
-	Packet p(PACKET_UDP_SERVER_REGISTER);
-	/* Packet is: WELCOME_MESSAGE, Version, server_port */
-	p.Send_string(NETWORK_MASTER_SERVER_WELCOME_MESSAGE);
-	p.Send_uint8 (NETWORK_MASTER_SERVER_VERSION);
-	p.Send_uint16(_network_server_port);
-	_udp_master_socket->SendPacket(&p, &out_addr);
+	if (!ThreadObject::New(NetworkUDPAdvertiseThread, NULL)) {
+		NetworkUDPAdvertiseThread(NULL);
+	}
 }
 
 void NetworkUDPInitialize()
 {
+	assert(_udp_client_socket == NULL && _udp_server_socket == NULL && _udp_master_socket == NULL);
+
+	_network_udp_mutex->BeginCritical();
 	_udp_client_socket = new ClientNetworkUDPSocketHandler();
 	_udp_server_socket = new ServerNetworkUDPSocketHandler();
 	_udp_master_socket = new MasterNetworkUDPSocketHandler();
 
 	_network_udp_server = false;
 	_network_udp_broadcast = 0;
+	_network_udp_mutex->EndCritical();
 }
 
 void NetworkUDPShutdown()
 {
 	NetworkUDPCloseAll();
 
+	_network_udp_mutex->BeginCritical();
 	delete _udp_client_socket;
 	delete _udp_server_socket;
 	delete _udp_master_socket;
+	_udp_client_socket = NULL;
+	_udp_server_socket = NULL;
+	_udp_master_socket = NULL;
+	_network_udp_mutex->EndCritical();
 }
 
 #endif /* ENABLE_NETWORK */
