@@ -1,8 +1,24 @@
-/* $Id: macos.h 11521 2007-11-25 14:43:16Z egladil $ */
+/* $Id: macos.h 15718 2009-03-15 00:32:18Z rubidium $ */
+
+/** @file macos.h Functions related to MacOS support. */
 
 #ifndef MACOS_H
 #define MACOS_H
 
+/* It would seem that to ensure backward compability we have to ensure that we have defined MAC_OS_X_VERSION_10_x everywhere */
+#ifndef MAC_OS_X_VERSION_10_3
+#define MAC_OS_X_VERSION_10_3 1030
+#endif
+
+#ifndef MAC_OS_X_VERSION_10_4
+#define MAC_OS_X_VERSION_10_4 1040
+#endif
+
+#ifndef MAC_OS_X_VERSION_10_5
+#define MAC_OS_X_VERSION_10_5 1050
+#endif
+
+
 /*
  * Functions to show the popup window
  * use ShowMacDialog when you want to control title, message and text on the button
@@ -15,8 +31,8 @@ void ShowMacDialog ( const char *title, const char *message, const char *buttonL
 void ShowMacAssertDialog ( const char *function, const char *file, const int line, const char *expression );
 void ShowMacErrorDialog(const char *error);
 
-// Since MacOS X users will never see an assert unless they started the game from a terminal
-// we're using a custom assert(e) macro.
+/* Since MacOS X users will never see an assert unless they started the game from a terminal
+ * we're using a custom assert(e) macro. */
 #undef assert
 
 #ifdef NDEBUG
@@ -67,4 +83,14 @@ static inline bool MacOSVersionIsAtLeast(long major, long minor, long bugfix)
 	return true;
 }
 
+/*
+ * OSX 10.3.9 has blessed us with a signal with unlikable side effects.
+ * The most problematic side effect is that it makes OpenTTD 'think' that
+ * it's running on 10.4.0 or higher and thus tries to link to functions
+ * that are only defined there. So now we'll remove all and any signal
+ * handling for OSX < 10.4 and 10.3.9 works as it should at the cost of
+ * not giving a useful error when savegame loading goes wrong.
+ */
+#define signal(sig, func) (MacOSVersionIsAtLeast(10, 4, 0) ? signal(sig, func) : NULL)
+
 #endif /* MACOS_H */
