@@ -1,4 +1,4 @@
-/* $Id: vehicle_base.h 13017 2008-05-08 17:30:02Z rubidium $ */
+/* $Id: vehicle_base.h 16482 2009-05-31 12:22:53Z rubidium $ */
 
 /** @file  vehicle_base.h Base class for all vehicles. */
 
@@ -11,15 +11,19 @@
 #include "road_type.h"
 #include "cargo_type.h"
 #include "direction_type.h"
-#include "window_type.h"
 #include "gfx_type.h"
 #include "command_type.h"
 #include "date_type.h"
-#include "player_type.h"
+#include "company_base.h"
+#include "company_type.h"
 #include "oldpool.h"
-#include "order.h"
+#include "order_base.h"
 #include "cargopacket.h"
 #include "texteff.hpp"
+#include "group_type.h"
+#include "engine_type.h"
+#include "order_func.h"
+#include "transport_type.h"
 
 /** Road vehicle states */
 enum RoadVehicleStates {
@@ -71,25 +75,33 @@ enum VehicleFlags {
 	VF_LOADING_FINISHED,
 	VF_CARGO_UNLOADING,
 	VF_BUILT_AS_PROTOTYPE,
-	VF_TIMETABLE_STARTED,  ///< Whether the vehicle has started running on the timetable yet.
-	VF_AUTOFILL_TIMETABLE, ///< Whether the vehicle should fill in the timetable automatically.
+	VF_TIMETABLE_STARTED,       ///< Whether the vehicle has started running on the timetable yet.
+	VF_AUTOFILL_TIMETABLE,      ///< Whether the vehicle should fill in the timetable automatically.
+	VF_AUTOFILL_PRES_WAIT_TIME, ///< Whether non-destructive auto-fill should preserve waiting times
 };
 
 struct VehicleRail {
+	/* Link between the two ends of a multiheaded engine */
+	Vehicle *other_multiheaded_part;
+
+	/* Cached wagon override spritegroup */
+	const struct SpriteGroup *cached_override;
+
 	uint16 last_speed; // NOSAVE: only used in UI
 	uint16 crash_anim_pos;
 
 	/* cached values, recalculated on load and each time a vehicle is added to/removed from the consist. */
-	uint16 cached_max_speed;  // max speed of the consist. (minimum of the max speed of all vehicles in the consist)
-	uint32 cached_power;      // total power of the consist.
-	bool cached_tilt;         // train can tilt; feature provides a bonus in curves
-	uint8 cached_veh_length;  // length of this vehicle in units of 1/8 of normal length, cached because this can be set by a callback
+	uint32 cached_power;        ///< total power of the consist.
+	uint16 cached_max_speed;    ///< max speed of the consist. (minimum of the max speed of all vehicles in the consist)
 	uint16 cached_total_length; ///< Length of the whole train, valid only for first engine.
+	uint8 cached_veh_length;    ///< length of this vehicle in units of 1/8 of normal length, cached because this can be set by a callback
+	bool cached_tilt;           ///< train can tilt; feature provides a bonus in curves
 
 	/* cached values, recalculated when the cargo on a train changes (in addition to the conditions above) */
-	uint32 cached_weight;     // total weight of the consist.
-	uint32 cached_veh_weight; // weight of the vehicle.
-	uint32 cached_max_te;     // max tractive effort of consist
+	uint32 cached_weight;     ///< total weight of the consist.
+	uint32 cached_veh_weight; ///< weight of the vehicle.
+	uint32 cached_max_te;     ///< max tractive effort of consist
+
 	/**
 	 * Position/type of visual effect.
 	 * bit 0 - 3 = position of effect relative to vehicle. (0 = front, 8 = centre, 15 = rear)
@@ -104,18 +116,11 @@ struct VehicleRail {
 	 * 0xffff == not in train */
 	EngineID first_engine;
 
+	uint16 flags;
 	TrackBitsByte track;
 	byte force_proceed;
 	RailTypeByte railtype;
 	RailTypes compatible_railtypes;
-
-	byte flags;
-
-	/* Link between the two ends of a multiheaded engine */
-	Vehicle *other_multiheaded_part;
-
-	/* Cached wagon override spritegroup */
-	const struct SpriteGroup *cached_override;
 };
 
 enum VehicleRailFlags {
@@ -137,8 +142,11 @@ enum VehicleRailFlags {
 	/* used to mark that electric train engine is allowed to run on normal rail */
 	VRF_EL_ENGINE_ALLOWED_NORMAL_RAIL = 6,
 
-	/* used for vehicle var 0xFE bit 8 (toggled each time the train is reversed) */
+	/* used for vehicle var 0xFE bit 8 (toggled each time the train is reversed, accurate for first vehicle only) */
 	VRF_TOGGLE_REVERSE = 7,
+
+	/* used to mark a train that can't get a path reservation */
+	VRF_TRAIN_STUCK    = 8,
 };
 
 struct VehicleAir {
@@ -167,7 +175,7 @@ struct VehicleRoad {
 	RoadTypes compatible_roadtypes;
 };
 
-struct VehicleSpecial {
+struct VehicleEffect {
 	uint16 animation_state;
 	byte animation_substate;
 };
@@ -186,98 +194,52 @@ DECLARE_OLD_POOL(Vehicle, Vehicle, 9, 125)
 /* Some declarations of functions, so we can make them friendly */
 struct SaveLoad;
 extern const SaveLoad *GetVehicleDescription(VehicleType vt);
-extern void AfterLoadVehicles(bool clear_te_id);
 struct LoadgameState;
 extern bool LoadOldVehicle(LoadgameState *ls, int num);
 
 struct Vehicle : PoolItem<Vehicle, VehicleID, &_Vehicle_pool>, BaseVehicle {
-	byte subtype;            // subtype (Filled with values from EffectVehicles/TrainSubTypes/AircraftSubTypes)
-
 private:
-	Vehicle *next;           // pointer to the next vehicle in the chain
-	Vehicle *previous;       // NOSAVE: pointer to the previous vehicle in the chain
-	Vehicle *first;          // NOSAVE: pointer to the first vehicle in the chain
-public:
-	friend const SaveLoad *GetVehicleDescription(VehicleType vt); // So we can use private/protected variables in the saveload code
-	friend void AfterLoadVehicles(bool clear_te_id);              // So we can set the previous and first pointers while loading
-	friend bool LoadOldVehicle(LoadgameState *ls, int num);       // So we can set the proper next pointer while loading
+	Vehicle *next;           ///< pointer to the next vehicle in the chain
+	Vehicle *previous;       ///< NOSAVE: pointer to the previous vehicle in the chain
+	Vehicle *first;          ///< NOSAVE: pointer to the first vehicle in the chain
 
-	Vehicle *depot_list;     // NOSAVE: linked list to tell what vehicles entered a depot during the last tick. Used by autoreplace
+	Vehicle *next_shared;     ///< pointer to the next vehicle that shares the order
+	Vehicle *previous_shared; ///< NOSAVE: pointer to the previous vehicle in the shared order chain
+public:
+	friend const SaveLoad *GetVehicleDescription(VehicleType vt); ///< So we can use private/protected variables in the saveload code
+	friend void AfterLoadVehicles(bool part_of_load);             ///< So we can set the previous and first pointers while loading
+	friend bool LoadOldVehicle(LoadgameState *ls, int num);       ///< So we can set the proper next pointer while loading
 
 	char *name;              ///< Name of vehicle
 
-	UnitID unitnumber;       // unit number, for display purposes only
-	PlayerByte owner;        // which player owns the vehicle?
+	TileIndex tile;          ///< Current tile index
 
-	TileIndex tile;          // Current tile index
-	TileIndex dest_tile;     // Heading for this tile
+	/**
+	 * Heading for this tile.
+	 * For airports and train stations this tile does not necessarily belong to the destination station,
+	 * but it can be used for heuristical purposes to estimate the distance.
+	 */
+	TileIndex dest_tile;
 
-	int32 x_pos;             // coordinates
-	int32 y_pos;
-	byte z_pos;
-	DirectionByte direction; // facing
+	Money profit_this_year;        ///< Profit this year << 8, low 8 bits are fract
+	Money profit_last_year;        ///< Profit last year << 8, low 8 bits are fract
+	Money value;
 
-	byte spritenum;          // currently displayed sprite index
-	                         // 0xfd == custom sprite, 0xfe == custom second head sprite
-	                         // 0xff == reserved for another custom sprite
-	uint16 cur_image;        // sprite number for this vehicle
-	byte sprite_width;       // width of vehicle sprite
-	byte sprite_height;      // height of vehicle sprite
-	byte z_height;           // z-height of vehicle sprite
-	int8 x_offs;             // x offset for vehicle sprite
-	int8 y_offs;             // y offset for vehicle sprite
-	EngineID engine_type;
-
-	TextEffectID fill_percent_te_id; // a text-effect id to a loading indicator object
-
-	/* for randomized variational spritegroups
-	 * bitmask used to resolve them; parts of it get reseeded when triggers
-	 * of corresponding spritegroups get matched */
-	byte random_bits;
-	byte waiting_triggers;   // triggers to be yet matched
-
-	uint16 max_speed;        // maximum speed
-	uint16 cur_speed;        // current speed
-	byte subspeed;           // fractional speed
-	byte acceleration;       // used by train & aircraft
-	byte progress;
-	uint32 motion_counter;
-
-	byte vehstatus;          // Status
-	StationID last_station_visited;
-
-	CargoID cargo_type;      // type of cargo this vehicle is carrying
-	uint16 cargo_cap;        // total capacity
-	byte cargo_subtype;      ///< Used for livery refits (NewGRF variations)
-	CargoList cargo;         ///< The cargo this vehicle is carrying
-
-
-	byte day_counter;        ///< Increased by one for each day
-	byte tick_counter;       ///< Increased by one for each tick
-	byte running_ticks;      ///< Number of ticks this vehicle was not stopped this day
-
-	/* Begin Order-stuff */
-	Order current_order;     ///< The current order (+ status, like: loading)
-	VehicleOrderID cur_order_index; ///< The index to the current order
-
-	Order *orders;           ///< Pointer to the first order for this vehicle
-	VehicleOrderID num_orders;      ///< How many orders there are in the list
-
-	Vehicle *next_shared;    ///< If not NULL, this points to the next vehicle that shared the order
-	Vehicle *prev_shared;    ///< If not NULL, this points to the prev vehicle that shared the order
-	/* End Order-stuff */
+	/* Used for timetabling. */
+	uint32 current_order_time;     ///< How many ticks have passed since this order started.
+	int32 lateness_counter;        ///< How many ticks late (or early if negative) this vehicle is.
 
 	/* Boundaries for the current position in the world and a next hash link.
 	 * NOSAVE: All of those can be updated with VehiclePositionChanged() */
-	int32 left_coord;
-	int32 top_coord;
-	int32 right_coord;
-	int32 bottom_coord;
+	Rect coord;
 	Vehicle *next_hash;
 	Vehicle *next_new_hash;
 	Vehicle **old_new_hash;
 
+	SpriteID colourmap; // NOSAVE: cached colour mapping
+
 	/* Related to age and service time */
+	Year build_year;
 	Date age;     // Age in days
 	Date max_age; // Maximum age
 	Date date_of_last_service;
@@ -288,34 +250,82 @@ public:
 	byte breakdown_delay;
 	byte breakdowns_since_last_service;
 	byte breakdown_chance;
-	Year build_year;
 
-	bool leave_depot_instantly; // NOSAVE: stores if the vehicle needs to leave the depot it just entered. Used by autoreplace
+	int32 x_pos;             // coordinates
+	int32 y_pos;
+	byte z_pos;
+	DirectionByte direction; // facing
 
+	OwnerByte owner;         // which company owns the vehicle?
+	byte spritenum;          // currently displayed sprite index
+	                         // 0xfd == custom sprite, 0xfe == custom second head sprite
+	                         // 0xff == reserved for another custom sprite
+	uint16 cur_image;        // sprite number for this vehicle
+	byte x_extent;           // x-extent of vehicle bounding box
+	byte y_extent;           // y-extent of vehicle bounding box
+	byte z_extent;           // z-extent of vehicle bounding box
+	int8 x_offs;             // x offset for vehicle sprite
+	int8 y_offs;             // y offset for vehicle sprite
+	EngineID engine_type;
+
+	TextEffectID fill_percent_te_id; // a text-effect id to a loading indicator object
+	UnitID unitnumber;       // unit number, for display purposes only
+
+	uint16 max_speed;        ///< maximum speed
+	uint16 cur_speed;        ///< current speed
+	byte subspeed;           ///< fractional speed
+	byte acceleration;       ///< used by train & aircraft
+	uint32 motion_counter;
+	byte progress;
+
+	/* for randomized variational spritegroups
+	 * bitmask used to resolve them; parts of it get reseeded when triggers
+	 * of corresponding spritegroups get matched */
+	byte random_bits;
+	byte waiting_triggers;   ///< triggers to be yet matched
+
+	StationID last_station_visited;
+
+	CargoID cargo_type;      ///< type of cargo this vehicle is carrying
+	byte cargo_subtype;      ///< Used for livery refits (NewGRF variations)
+	uint16 cargo_cap;        ///< total capacity
+	CargoList cargo;         ///< The cargo this vehicle is carrying
+
+	byte day_counter;        ///< Increased by one for each day
+	byte tick_counter;       ///< Increased by one for each tick
+	byte running_ticks;      ///< Number of ticks this vehicle was not stopped this day
+
+	byte vehstatus;                 ///< Status
+	Order current_order;            ///< The current order (+ status, like: loading)
+	VehicleOrderID cur_order_index; ///< The index to the current order
+
+	union {
+		OrderList *list;              ///< Pointer to the order list for this vehicle
+		Order     *old;               ///< Only used during conversion of old save games
+	} orders;
+
+	byte vehicle_flags;             ///< Used for gradual loading and other miscellaneous things (@see VehicleFlags enum)
 	uint16 load_unload_time_rem;
-	byte vehicle_flags;         // Used for gradual loading and other miscellaneous things (@see VehicleFlags enum)
 
-	Money profit_this_year;        ///< Profit this year << 8, low 8 bits are fract
-	Money profit_last_year;        ///< Profit last year << 8, low 8 bits are fract
-	Money value;
+	GroupID group_id;               ///< Index of group Pool array
 
-	GroupID group_id;              ///< Index of group Pool array
-
-	/* Used for timetabling. */
-	uint32 current_order_time;     ///< How many ticks have passed since this order started.
-	int32 lateness_counter;        ///< How many ticks late (or early if negative) this vehicle is.
-
-	SpriteID colormap; // NOSAVE: cached color mapping
+	byte subtype;                   ///< subtype (Filled with values from EffectVehicles/TrainSubTypes/AircraftSubTypes)
 
 	union {
 		VehicleRail rail;
 		VehicleAir air;
 		VehicleRoad road;
-		VehicleSpecial special;
+		VehicleEffect effect;
 		VehicleDisaster disaster;
 		VehicleShip ship;
 	} u;
 
+	/* cached oftenly queried NewGRF values */
+	uint8 cache_valid;   ///< Whether the caches are valid
+	uint32 cached_var40; ///< Cache for NewGRF var 40
+	uint32 cached_var41; ///< Cache for NewGRF var 41
+	uint32 cached_var42; ///< Cache for NewGRF var 42
+	uint32 cached_var43; ///< Cache for NewGRF var 43
 
 	/**
 	 * Allocates a lot of vehicles.
@@ -347,7 +357,7 @@ public:
 	 * Get a string 'representation' of the vehicle type.
 	 * @return the string representation.
 	 */
-	virtual const char* GetTypeString() const { return "base vehicle"; }
+	virtual const char *GetTypeString() const { return "base vehicle"; }
 
 	/**
 	 * Marks the vehicles to be redrawn and updates cached variables
@@ -372,11 +382,6 @@ public:
 	 */
 	virtual ExpensesType GetExpenseType(bool income) const { return EXPENSES_OTHER; }
 
-	/**
-	 * Invalidates the vehicle list window of this type of vehicle
-	 */
-	virtual WindowClass GetVehicleListWindowClass() const { return WC_NONE; }
-
 	/**
 	 * Play the sound associated with leaving the station
 	 */
@@ -392,16 +397,36 @@ public:
 	 * @param direction the direction the vehicle is facing
 	 * @return the sprite for the given vehicle in the given direction
 	 */
-	virtual int GetImage(Direction direction) const { return 0; }
+	virtual SpriteID GetImage(Direction direction) const { return 0; }
 
 	/**
-	 * Gets the speed in mph that can be sent into SetDParam for string processing.
+	 * Invalidates cached NewGRF variables
+	 * @see InvalidateNewGRFCacheOfChain
+	 */
+	FORCEINLINE void InvalidateNewGRFCache()
+	{
+		this->cache_valid = 0;
+	}
+
+	/**
+	 * Invalidates cached NewGRF variables of all vehicles in the chain (after the current vehicle)
+	 * @see InvalidateNewGRFCache
+	 */
+	FORCEINLINE void InvalidateNewGRFCacheOfChain()
+	{
+		for (Vehicle *u = this; u != NULL; u = u->Next()) {
+			u->InvalidateNewGRFCache();
+		}
+	}
+
+	/**
+	 * Gets the speed in km-ish/h that can be sent into SetDParam for string processing.
 	 * @return the vehicle's speed
 	 */
 	virtual int GetDisplaySpeed() const { return 0; }
 
 	/**
-	 * Gets the maximum speed in mph that can be sent into SetDParam for string processing.
+	 * Gets the maximum speed in km-ish/h that can be sent into SetDParam for string processing.
 	 * @return the vehicle's maximum speed
 	 */
 	virtual int GetDisplayMaxSpeed() const { return 0; }
@@ -478,14 +503,79 @@ public:
 	 */
 	inline Vehicle *First() const { return this->first; }
 
+
+	/**
+	 * Get the first order of the vehicles order list.
+	 * @return first order of order list.
+	 */
+	inline Order *GetFirstOrder() const { return (this->orders.list == NULL) ? NULL : this->orders.list->GetFirstOrder(); }
+
+	/**
+	 * Adds this vehicle to a shared vehicle chain.
+	 * @param shared_chain a vehicle of the chain with shared vehicles.
+	 * @pre !this->IsOrderListShared()
+	 */
+	void AddToShared(Vehicle *shared_chain);
+
+	/**
+	 * Removes the vehicle from the shared order list.
+	 */
+	void RemoveFromShared();
+
+	/**
+	 * Get the next vehicle of the shared vehicle chain.
+	 * @return the next shared vehicle or NULL when there isn't a next vehicle.
+	 */
+	inline Vehicle *NextShared() const { return this->next_shared; }
+
+	/**
+	 * Get the previous vehicle of the shared vehicle chain
+	 * @return the previous shared vehicle or NULL when there isn't a previous vehicle.
+	 */
+	inline Vehicle *PreviousShared() const { return this->previous_shared; }
+
+	/**
+	 * Get the first vehicle of this vehicle chain.
+	 * @return the first vehicle of the chain.
+	 */
+	inline Vehicle *FirstShared() const { return (this->orders.list == NULL) ? this->First() : this->orders.list->GetFirstSharedVehicle(); }
+
 	/**
 	 * Check if we share our orders with another vehicle.
-	 * This is done by checking the previous and next pointers in the shared chain.
 	 * @return true if there are other vehicles sharing the same order
 	 */
-	inline bool IsOrderListShared() const { return this->next_shared != NULL || this->prev_shared != NULL; };
+	inline bool IsOrderListShared() const { return this->orders.list != NULL && this->orders.list->IsShared(); }
 
-	bool NeedsAutorenewing(const Player *p) const;
+	/**
+	 * Get the number of orders this vehicle has.
+	 * @return the number of orders this vehicle has.
+	 */
+	inline VehicleOrderID GetNumOrders() const { return (this->orders.list == NULL) ? 0 : this->orders.list->GetNumOrders(); }
+
+	/**
+	 * Copy certain configurations and statistics of a vehicle after successful autoreplace/renew
+	 * The function shall copy everything that cannot be copied by a command (like orders / group etc),
+	 * and that shall not be resetted for the new vehicle.
+	 * @param src The old vehicle
+	 */
+	inline void CopyVehicleConfigAndStatistics(const Vehicle *src)
+	{
+		this->unitnumber = src->unitnumber;
+
+		this->cur_order_index = src->cur_order_index;
+		this->current_order = src->current_order;
+		this->dest_tile  = src->dest_tile;
+
+		this->profit_this_year = src->profit_this_year;
+		this->profit_last_year = src->profit_last_year;
+
+		this->current_order_time = src->current_order_time;
+		this->lateness_counter = src->lateness_counter;
+
+		this->service_interval = src->service_interval;
+	}
+
+	bool NeedsAutorenewing(const Company *c) const;
 
 	/**
 	 * Check if the vehicle needs to go to a depot in near future (if a opportunity presents itself) for service or replacement.
@@ -501,33 +591,33 @@ public:
 	 * @return true if the current order should be interupted.
 	 */
 	bool NeedsAutomaticServicing() const;
-};
 
-/**
- * This class 'wraps' Vehicle; you do not actually instantiate this class.
- * You create a Vehicle using AllocateVehicle, so it is added to the pool
- * and you reinitialize that to a Train using:
- *   v = new (v) Train();
- *
- * As side-effect the vehicle type is set correctly.
- *
- * A special vehicle is one of the following:
- *  - smoke
- *  - electric sparks for trains
- *  - explosions
- *  - bulldozer (road works)
- *  - bubbles (industry)
- */
-struct SpecialVehicle : public Vehicle {
-	/** Initializes the Vehicle to a special vehicle */
-	SpecialVehicle() { this->type = VEH_SPECIAL; }
+	/**
+	 * Determine the location for the station where the vehicle goes to next.
+	 * Things done for example are allocating slots in a road stop or exact
+	 * location of the platform is determined for ships.
+	 * @param station the station to make the next location of the vehicle.
+	 * @return the location (tile) to aim for.
+	 */
+	virtual TileIndex GetOrderStationLocation(StationID station) { return INVALID_TILE; }
 
-	/** We want to 'destruct' the right class. */
-	virtual ~SpecialVehicle() {}
+	/**
+	 * Find the closest depot for this vehicle and tell us the location,
+	 * DestinationID and whether we should reverse.
+	 * @param location    where do we go to?
+	 * @param destination what hangar do we go to?
+	 * @param reverse     should the vehicle be reversed?
+	 * @return true if a depot could be found.
+	 */
+	virtual bool FindClosestDepot(TileIndex *location, DestinationID *destination, bool *reverse) { return false; }
 
-	const char *GetTypeString() const { return "special vehicle"; }
-	void UpdateDeltaXY(Direction direction);
-	void Tick();
+	/**
+	 * Send this vehicle to the depot using the given command(s).
+	 * @param flags   the command flags (like execute and such).
+	 * @param command the command to execute.
+	 * @return the cost of the depot action.
+	 */
+	CommandCost SendToDepot(DoCommandFlag flags, DepotCommand command);
 };
 
 /**
@@ -569,9 +659,6 @@ struct InvalidVehicle : public Vehicle {
 	void Tick() {}
 };
 
-#define BEGIN_ENUM_WAGONS(v) do {
-#define END_ENUM_WAGONS(v) } while ((v = v->Next()) != NULL);
-
 static inline VehicleID GetMaxVehicleIndex()
 {
 	/* TODO - This isn't the real content of the function, but
@@ -600,47 +687,37 @@ static inline bool IsValidVehicleID(uint index)
 	return index < GetVehiclePoolSize() && GetVehicle(index)->IsValid();
 }
 
+
+/** Generates sequence of free UnitID numbers */
+struct FreeUnitIDGenerator {
+	bool *cache;  ///< array of occupied unit id numbers
+	UnitID maxid; ///< maximum ID at the moment of constructor call
+	UnitID curid; ///< last ID returned ; 0 if none
+
+	/** Initializes the structure. Vehicle unit numbers are supposed not to change after
+	 * struct initialization, except after each call to this->NextID() the returned value
+	 * is assigned to a vehicle.
+	 * @param type type of vehicle
+	 * @param owner owner of vehicles
+	 */
+	FreeUnitIDGenerator(VehicleType type, CompanyID owner);
+
+	/** Returns next free UnitID. Supposes the last returned value was assigned to a vehicle. */
+	UnitID NextID();
+
+	/** Releases allocated memory */
+	~FreeUnitIDGenerator() { free(this->cache); }
+};
+
 /* Returns order 'index' of a vehicle or NULL when it doesn't exists */
-static inline Order *GetVehicleOrder(const Vehicle *v, int index)
-{
-	Order *order = v->orders;
-
-	if (index < 0) return NULL;
-
-	while (order != NULL && index-- > 0)
-		order = order->next;
-
-	return order;
-}
+static inline Order *GetVehicleOrder(const Vehicle *v, int index) { return (v->orders.list == NULL) ? NULL : v->orders.list->GetOrderAt(index); }
 
 /**
  * Returns the last order of a vehicle, or NULL if it doesn't exists
  * @param v Vehicle to query
  * @return last order of a vehicle, if available
  */
-static inline Order *GetLastVehicleOrder(const Vehicle *v)
-{
-	Order *order = v->orders;
-
-	if (order == NULL) return NULL;
-
-	while (order->next != NULL)
-		order = order->next;
-
-	return order;
-}
-
-/** Get the first vehicle of a shared-list, so we only have to walk forwards
- * @param v Vehicle to query
- * @return first vehicle of a shared-list
- */
-static inline Vehicle *GetFirstVehicleFromSharedList(const Vehicle *v)
-{
-	Vehicle *u = (Vehicle *)v;
-	while (u->prev_shared != NULL) u = u->prev_shared;
-
-	return u;
-}
+static inline Order *GetLastVehicleOrder(const Vehicle *v) { return (v->orders.list == NULL) ? NULL : v->orders.list->GetLastOrder(); }
 
 /**
  * Returns the Trackdir on which the vehicle is currently located.
@@ -653,8 +730,10 @@ static inline Vehicle *GetFirstVehicleFromSharedList(const Vehicle *v)
  * For other vehicles types, or vehicles with no clear trackdir (such as those
  * in depots), returns 0xFF.
  */
-Trackdir GetVehicleTrackdir(const Vehicle* v);
+Trackdir GetVehicleTrackdir(const Vehicle *v);
 
 void CheckVehicle32Day(Vehicle *v);
 
+static const int32 INVALID_COORD = 0x7fffffff;
+
 #endif /* VEHICLE_BASE_H */
