@@ -1,4 +1,4 @@
-/* $Id: fullscreen.mm 11718 2007-12-29 05:15:13Z egladil $ */
+/* $Id: fullscreen.mm 15711 2009-03-14 18:16:29Z rubidium $ */
 
 /******************************************************************************
  *                             Cocoa video driver                             *
@@ -77,7 +77,7 @@ struct OTTD_QuartzGammaTable {
 
 
 
-uint QZ_ListModes(OTTD_Point* modes, uint max_modes, CGDirectDisplayID display_id, int display_depth)
+uint QZ_ListModes(OTTD_Point *modes, uint max_modes, CGDirectDisplayID display_id, int display_depth)
 {
 	CFArrayRef mode_list;
 	CFIndex num_modes;
@@ -155,14 +155,23 @@ uint QZ_ListModes(OTTD_Point* modes, uint max_modes, CGDirectDisplayID display_i
 	return count;
 }
 
+/* Small function to test if the main display can display 8 bpp in fullscreen */
+bool QZ_CanDisplay8bpp()
+{
+	OTTD_Point p;
+
+	/* We want to know if 8 bpp is possible in fullscreen and not anything about resolutions.
+	 * Because of this we want to fill a list of 1 resolution of 8 bpp on display 0 (main) and return if we found one. */
+	return QZ_ListModes(&p, 1, 0, 8);
+}
 
 class FullscreenSubdriver: public CocoaSubdriver {
 	int                display_width;
 	int                display_height;
 	int                display_depth;
 	int                screen_pitch;
-	void*              screen_buffer;
-	void*              pixel_buffer;
+	void              *screen_buffer;
+	void              *pixel_buffer;
 
 	CGDirectDisplayID  display_id;         /* 0 == main display (only support single display) */
 	CFDictionaryRef    cur_mode;           /* current mode of the display */
@@ -178,7 +187,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 	 * Fade the display from normal to black
 	 * Save gamma tables for fade back to normal
 	 */
-	uint32 FadeGammaOut(OTTD_QuartzGammaTable* table)
+	uint32 FadeGammaOut(OTTD_QuartzGammaTable *table)
 	{
 		CGGammaValue redTable[QZ_GAMMA_TABLE_SIZE];
 		CGGammaValue greenTable[QZ_GAMMA_TABLE_SIZE];
@@ -223,7 +232,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 	/* Fade the display from black to normal
 	 * Restore previously saved gamma values
 	 */
-	uint32 FadeGammaIn(const OTTD_QuartzGammaTable* table)
+	uint32 FadeGammaIn(const OTTD_QuartzGammaTable *table)
 	{
 		CGGammaValue redTable[QZ_GAMMA_TABLE_SIZE];
 		CGGammaValue greenTable[QZ_GAMMA_TABLE_SIZE];
@@ -370,7 +379,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 		 * As a result, coordinate translation produces incorrect results.
 		 * We can hack around this bug by setting the screen rect ourselves.
 		 * This hack should be removed if/when the bug is fixed.
-		*/
+		 */
 		screen_rect = NSMakeRect(0, 0, display_width, display_height);
 		[ [ NSScreen mainScreen ] setFrame:screen_rect ];
 
@@ -461,8 +470,8 @@ public:
 
 	virtual void Draw()
 	{
-		const uint8* src   = (uint8*) pixel_buffer;
-		uint8* dst         = (uint8*) screen_buffer;
+		const uint8 *src   = (uint8*) pixel_buffer;
+		uint8 *dst         = (uint8*) screen_buffer;
 		uint pitch         = screen_pitch;
 		uint width         = display_width;
 		uint num_dirty     = num_dirty_rects;
@@ -527,7 +536,7 @@ public:
 		CGDisplaySetPalette(display_id, palette);
 	}
 
-	virtual uint ListModes(OTTD_Point* modes, uint max_modes)
+	virtual uint ListModes(OTTD_Point *modes, uint max_modes)
 	{
 		return QZ_ListModes(modes, max_modes, display_id, display_depth);
 	}
@@ -567,10 +576,10 @@ public:
 	}
 
 	/*
-		Convert local coordinate to window server (CoreGraphics) coordinate.
-		In fullscreen mode this just means copying the coords.
-	*/
-	virtual CGPoint PrivateLocalToCG(NSPoint* p)
+	 * Convert local coordinate to window server (CoreGraphics) coordinate.
+	 * In fullscreen mode this just means copying the coords.
+	 */
+	virtual CGPoint PrivateLocalToCG(NSPoint *p)
 	{
 		CGPoint cgp;
 
