@@ -1,31 +1,30 @@
-/* $Id: win32.cpp 14426 2008-10-01 11:48:57Z rubidium $ */
+/* $Id: win32.cpp 16309 2009-05-15 10:17:00Z rubidium $ */
 
 /** @file win32.cpp Implementation of MS Windows system calls */
 
 #include "stdafx.h"
 #include "openttd.h"
 #include "debug.h"
-#include "saveload.h"
+#include "saveload/saveload.h"
 #include "gfx_func.h"
 #include "textbuf_gui.h"
-#include "fileio.h"
+#include "fileio_func.h"
+#include "fios.h"
+#include "rev.h"
 #include <windows.h>
 #include <winnt.h>
 #include <wininet.h>
-#include <io.h>
 #include <fcntl.h>
-#include <shlobj.h> // SHGetFolderPath
+#include <shlobj.h> /* SHGetFolderPath */
 #include "variables.h"
 #include "win32.h"
-#include "fios.h" // opendir/readdir/closedir
-#include "fileio.h"
 #include "core/alloc_func.hpp"
 #include "functions.h"
 #include "core/random_func.hpp"
 #include "core/bitmath_func.hpp"
 #include "string_func.h"
+#include "gamelog.h"
 #include <ctype.h>
-#include <tchar.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -36,10 +35,6 @@
 
 static bool _has_console;
 
-#if defined(__MINGW32__)
-	#include <stdint.h>
-#endif
-
 static bool cursor_visible = true;
 
 bool MyShowCursor(bool show)
@@ -65,7 +60,7 @@ bool LoadLibraryList(Function proc[], const char *dll)
 		for (;;) {
 			FARPROC p;
 
-			while (*dll++ != '\0');
+			while (*dll++ != '\0') { /* Nothing */ }
 			if (*dll == '\0') break;
 #if defined(WINCE)
 			p = GetProcAddress(lib, MB_TO_WIDE(dll));
@@ -95,14 +90,14 @@ void SetExceptionString(const char *s, ...)
 }
 #endif
 
-void ShowOSErrorBox(const char *buf)
+void ShowOSErrorBox(const char *buf, bool system)
 {
 	MyShowCursor(true);
 	MessageBox(GetActiveWindow(), MB_TO_WIDE(buf), _T("Error!"), MB_ICONSTOP);
 
 /* if exception tracker is enabled, we crash here to let the exception handler handle it. */
 #if defined(WIN32_EXCEPTION_TRACKER) && !defined(_DEBUG)
-	if (*buf == '!') {
+	if (system) {
 		_exception_string = buf;
 		*(byte*)0 = 0;
 	}
@@ -153,7 +148,7 @@ static uint32 CalcCRC(byte *data, uint size, uint32 crc)
 static void GetFileInfo(DebugFileInfo *dfi, const TCHAR *filename)
 {
 	HANDLE file;
-	memset(dfi, 0, sizeof(dfi));
+	memset(dfi, 0, sizeof(*dfi));
 
 	file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, 0);
 	if (file != INVALID_HANDLE_VALUE) {
@@ -180,14 +175,14 @@ static void GetFileInfo(DebugFileInfo *dfi, const TCHAR *filename)
 }
 
 
-static char *PrintModuleInfo(char *output, HMODULE mod)
+static char *PrintModuleInfo(char *output, const char *last, HMODULE mod)
 {
 	TCHAR buffer[MAX_PATH];
 	DebugFileInfo dfi;
 
 	GetModuleFileName(mod, buffer, MAX_PATH);
 	GetFileInfo(&dfi, buffer);
-	output += sprintf(output, " %-20s handle: %p size: %d crc: %.8X date: %d-%.2d-%.2d %.2d:%.2d:%.2d\r\n",
+	output += seprintf(output, last, " %-20s handle: %p size: %d crc: %.8X date: %d-%.2d-%.2d %.2d:%.2d:%.2d\r\n",
 		WIDE_TO_MB(buffer),
 		mod,
 		dfi.size,
@@ -202,7 +197,7 @@ static char *PrintModuleInfo(char *output, HMODULE mod)
 	return output;
 }
 
-static char *PrintModuleList(char *output)
+static char *PrintModuleList(char *output, const char *last)
 {
 	BOOL (WINAPI *EnumProcessModules)(HANDLE, HMODULE*, DWORD, LPDWORD);
 
@@ -210,21 +205,20 @@ static char *PrintModuleList(char *output)
 		HMODULE modules[100];
 		DWORD needed;
 		BOOL res;
-		int count, i;
 
 		HANDLE proc = OpenProcess(PROCESS_ALL_ACCESS, FALSE, GetCurrentProcessId());
 		if (proc != NULL) {
 			res = EnumProcessModules(proc, modules, sizeof(modules), &needed);
 			CloseHandle(proc);
 			if (res) {
-				count = min(needed / sizeof(HMODULE), lengthof(modules));
+				size_t count = min(needed / sizeof(HMODULE), lengthof(modules));
 
-				for (i = 0; i != count; i++) output = PrintModuleInfo(output, modules[i]);
+				for (size_t i = 0; i != count; i++) output = PrintModuleInfo(output, last, modules[i]);
 				return output;
 			}
 		}
 	}
-	output = PrintModuleInfo(output, NULL);
+	output = PrintModuleInfo(output, last, NULL);
 	return output;
 }
 
@@ -240,8 +234,15 @@ static const TCHAR _save_succeeded[] =
 	_T("Be aware that critical parts of the internal game state may have become ")
 	_T("corrupted. The saved game is not guaranteed to work.");
 
+static const TCHAR _emergency_crash[] =
+	_T("A serious fault condition occured in the game. The game will shut down.\n")
+	_T("As you loaded an emergency savegame no crash information will be generated.\n");
+
 static bool EmergencySave()
 {
+	GamelogStartAction(GLAT_EMERGENCY);
+	GamelogEmergency();
+	GamelogStopAction();
 	SaveOrLoad("crash.sav", SL_SAVE, BASE_DIR);
 	return true;
 }
@@ -250,7 +251,7 @@ static bool EmergencySave()
 #if 0
 
 struct WinInetProcs {
-	HINTERNET (WINAPI *InternetOpen)(LPCTSTR,DWORD, LPCTSTR, LPCTSTR, DWORD);
+	HINTERNET (WINAPI *InternetOpen)(LPCTSTR, DWORD, LPCTSTR, LPCTSTR, DWORD);
 	HINTERNET (WINAPI *InternetConnect)(HINTERNET, LPCTSTR, INTERNET_PORT, LPCTSTR, LPCTSTR, DWORD, DWORD, DWORD);
 	HINTERNET (WINAPI *HttpOpenRequest)(HINTERNET, LPCTSTR, LPCTSTR, LPCTSTR, LPCTSTR, LPCTSTR *, DWORD, DWORD);
 	BOOL (WINAPI *HttpSendRequest)(HINTERNET, LPCTSTR, DWORD, LPVOID, DWORD);
@@ -402,9 +403,9 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 		} return TRUE;
 		case WM_COMMAND:
 			switch (wParam) {
-				case 12: /* Close */
+				case 12: // Close
 					ExitProcess(0);
-				case 13: /* Emergency save */
+				case 13: // Emergency save
 					if (DoEmergencySave(wnd)) {
 						MessageBox(wnd, _save_succeeded, _T("Save successful"), MB_ICONINFORMATION);
 					} else {
@@ -413,7 +414,7 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 					break;
 /* Disable the crash-save submit code as it's not used */
 #if 0
-				case 14: { /* Submit crash report */
+				case 14: { // Submit crash report
 					const TCHAR *s;
 
 					SetCursor(LoadCursor(NULL, IDC_WAIT));
@@ -424,10 +425,10 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 						break;
 					}
 
-					// try to submit emergency savegame
+					/* try to submit emergency savegame */
 					if (_did_emerg_save || DoEmergencySave(wnd)) SubmitFile(wnd, _T("crash.sav"));
 
-					// try to submit the autosaved game
+					/* try to submit the autosaved game */
 					if (_opt.autosave) {
 						TCHAR buf[40];
 						_sntprintf(buf, lengthof(buf), _T("autosave%d.sav"), (_autosave_ctr - 1) & 3);
@@ -438,7 +439,7 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 					MessageBox(wnd, _T("Crash report submitted. Thank you."), _T("Crash Report"), MB_ICONINFORMATION);
 				} break;
 #endif /* Disabled crash-submit procedures */
-				case 15: /* Expand window to show crash-message */
+				case 15: // Expand window to show crash-message
 					_expanded ^= 1;
 					SetWndSize(wnd, _expanded);
 					break;
@@ -459,44 +460,61 @@ static void Handler2()
 
 extern bool CloseConsoleLogIfActive();
 
+static HANDLE _file_crash_log;
+
+static void GamelogPrintCrashLogProc(const char *s)
+{
+	DWORD num_written;
+	WriteFile(_file_crash_log, s, (DWORD)strlen(s), &num_written, NULL);
+	WriteFile(_file_crash_log, "\r\n", (DWORD)strlen("\r\n"), &num_written, NULL);
+}
+
+/** Amount of output for the execption handler. */
+static const int EXCEPTION_OUTPUT_SIZE = 8192;
+
 static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 {
-	extern const char _openttd_revision[];
 	char *output;
 	static bool had_exception = false;
 
 	if (had_exception) ExitProcess(0);
+	if (GamelogTestEmergency()) {
+		MessageBox(NULL, _emergency_crash, _T("Fatal Application Failure"), MB_ICONERROR);
+		ExitProcess(0);
+	}
 	had_exception = true;
 
 	_ident = GetTickCount(); // something pretty unique
 
-	MakeCRCTable((uint32*)alloca(256 * sizeof(uint32)));
-	_crash_msg = output = (char*)LocalAlloc(LMEM_FIXED, 8192);
+	MakeCRCTable(AllocaM(uint32, 256));
+	_crash_msg = output = (char*)LocalAlloc(LMEM_FIXED, EXCEPTION_OUTPUT_SIZE);
+	const char *last = output + EXCEPTION_OUTPUT_SIZE - 1;
 
 	{
 		SYSTEMTIME time;
 		GetLocalTime(&time);
-		output += sprintf(output,
+		output += seprintf(output, last,
 			"*** OpenTTD Crash Report ***\r\n"
 			"Date: %d-%.2d-%.2d %.2d:%.2d:%.2d\r\n"
-			"Build: %s built on " __DATE__ " " __TIME__ "\r\n",
+			"Build: %s (%d) built on " __DATE__ " " __TIME__ "\r\n",
 			time.wYear,
 			time.wMonth,
 			time.wDay,
 			time.wHour,
 			time.wMinute,
 			time.wSecond,
-			_openttd_revision
+			_openttd_revision,
+			_openttd_revision_modified
 		);
 	}
 
 	if (_exception_string)
-		output += sprintf(output, "Reason: %s\r\n", _exception_string);
+		output += seprintf(output, last, "Reason: %s\r\n", _exception_string);
 
-	output += sprintf(output, "Language: %s\r\n", _dynlang.curr_file);
+	output += seprintf(output, last, "Language: %s\r\n", _dynlang.curr_file);
 
 #ifdef _M_AMD64
-	output += sprintf(output, "Exception %.8X at %.16IX\r\n"
+	output += seprintf(output, last, "Exception %.8X at %.16IX\r\n"
 		"Registers:\r\n"
 		"RAX: %.16llX RBX: %.16llX RCX: %.16llX RDX: %.16llX\r\n"
 		"RSI: %.16llX RDI: %.16llX RBP: %.16llX RSP: %.16llX\r\n"
@@ -526,7 +544,7 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 		ep->ContextRecord->EFlags
 	);
 #else
-	output += sprintf(output, "Exception %.8X at %.8p\r\n"
+	output += seprintf(output, last, "Exception %.8X at %.8p\r\n"
 		"Registers:\r\n"
 		" EAX: %.8X EBX: %.8X ECX: %.8X EDX: %.8X\r\n"
 		" ESI: %.8X EDI: %.8X EBP: %.8X ESP: %.8X\r\n"
@@ -556,13 +574,13 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 		int i;
 		for (i = 0; i != 24; i++) {
 			if (IsBadReadPtr(b, 1)) {
-				output += sprintf(output, " ??"); // OCR: WAS: , 0);
+				output += seprintf(output, last, " ??"); // OCR: WAS: , 0);
 			} else {
-				output += sprintf(output, " %.2X", *b);
+				output += seprintf(output, last, " %.2X", *b);
 			}
 			b++;
 		}
-		output += sprintf(output,
+		output += seprintf(output, last,
 			"\r\n"
 			"\r\nStack trace: \r\n"
 		);
@@ -578,35 +596,33 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 		for (j = 0; j != 24; j++) {
 			for (i = 0; i != 8; i++) {
 				if (IsBadReadPtr(b, sizeof(uint32))) {
-					output += sprintf(output, " ????????"); //OCR: WAS - , 0);
+					output += seprintf(output, last, " ????????"); // OCR: WAS - , 0);
 				} else {
-					output += sprintf(output, " %.8X", *b);
+					output += seprintf(output, last, " %.8X", *b);
 				}
 				b++;
 			}
-			output += sprintf(output, "\r\n");
+			output += seprintf(output, last, "\r\n");
 		}
 	}
 
-	output += sprintf(output, "\r\nModule information:\r\n");
-	output = PrintModuleList(output);
+	output += seprintf(output, last, "\r\nModule information:\r\n");
+	output = PrintModuleList(output, last);
 
 	{
 		_OSVERSIONINFOA os;
 		os.dwOSVersionInfoSize = sizeof(os);
 		GetVersionExA(&os);
-		output += sprintf(output, "\r\nSystem information:\r\n"
-			" Windows version %d.%d %d %s\r\n",
+		output += seprintf(output, last, "\r\nSystem information:\r\n"
+			" Windows version %d.%d %d %s\r\n\r\n",
 			os.dwMajorVersion, os.dwMinorVersion, os.dwBuildNumber, os.szCSDVersion);
 	}
 
-	{
-		HANDLE file = CreateFile(_T("crash.log"), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
+	_file_crash_log = CreateFile(_T("crash.log"), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
+
+	if (_file_crash_log != INVALID_HANDLE_VALUE) {
 		DWORD num_written;
-		if (file != INVALID_HANDLE_VALUE) {
-			WriteFile(file, _crash_msg, output - _crash_msg, &num_written, NULL);
-			CloseHandle(file);
-		}
+		WriteFile(_file_crash_log, _crash_msg, output - _crash_msg, &num_written, NULL);
 	}
 
 #if !defined(_DEBUG)
@@ -644,6 +660,11 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 	}
 #endif
 
+	if (_file_crash_log != INVALID_HANDLE_VALUE) {
+		GamelogPrint(&GamelogPrintCrashLogProc);
+		CloseHandle(_file_crash_log);
+	}
+
 	/* Close any possible log files */
 	CloseConsoleLogIfActive();
 
@@ -663,13 +684,13 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 }
 
 #ifdef _M_AMD64
-extern "C" void *_get_save_esp();
+extern "C" void *_get_safe_esp();
 #endif
 
 static void Win32InitializeExceptions()
 {
 #ifdef _M_AMD64
-	_safe_esp = _get_save_esp();
+	_safe_esp = _get_safe_esp();
 #else
 	_asm {
 		mov _safe_esp, esp
@@ -788,23 +809,23 @@ void FiosGetDrives()
 {
 #if defined(WINCE)
 	/* WinCE only knows one drive: / */
-	FiosItem *fios = FiosAlloc();
+	FiosItem *fios = _fios_items.Append();
 	fios->type = FIOS_TYPE_DRIVE;
 	fios->mtime = 0;
 	snprintf(fios->name, lengthof(fios->name), PATHSEP "");
-	ttd_strlcpy(fios->title, fios->name, lengthof(fios->title));
+	strecpy(fios->title, fios->name, lastof(fios->title));
 #else
 	TCHAR drives[256];
 	const TCHAR *s;
 
 	GetLogicalDriveStrings(lengthof(drives), drives);
 	for (s = drives; *s != '\0';) {
-		FiosItem *fios = FiosAlloc();
+		FiosItem *fios = _fios_items.Append();
 		fios->type = FIOS_TYPE_DRIVE;
 		fios->mtime = 0;
 		snprintf(fios->name, lengthof(fios->name),  "%c:", s[0] & 0xFF);
-		ttd_strlcpy(fios->title, fios->name, lengthof(fios->title));
-		while (*s++ != '\0');
+		strecpy(fios->title, fios->name, lastof(fios->title));
+		while (*s++ != '\0') { /* Nothing */ }
 	}
 #endif
 }
@@ -832,7 +853,7 @@ bool FiosIsHiddenFile(const struct dirent *ent)
 	return (ent->dir->fd.dwFileAttributes & (FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM)) != 0;
 }
 
-bool FiosGetDiskFreeSpace(const char *path, uint32 *tot)
+bool FiosGetDiskFreeSpace(const char *path, uint64 *tot)
 {
 	UINT sem = SetErrorMode(SEM_FAILCRITICALERRORS);  // disable 'no-disk' message box
 	bool retval = false;
@@ -841,7 +862,7 @@ bool FiosGetDiskFreeSpace(const char *path, uint32 *tot)
 
 	_sntprintf(root, lengthof(root), _T("%c:") _T(PATHSEP), path[0]);
 	if (tot != NULL && GetDiskFreeSpace(root, &spc, &bps, &nfc, &tnc)) {
-		*tot = ((spc * bps) * (uint64)nfc) >> 20;
+		*tot = ((spc * bps) * (uint64)nfc);
 		retval = true;
 	}
 
@@ -923,12 +944,12 @@ void ShowInfo(const char *str)
 	} else {
 		bool old;
 #if defined(UNICODE)
-			/* We need to put the text in a seperate buffer because the default
-			 * buffer in MB_TO_WIDE might not be large enough (512 chars) */
-			wchar_t help_msgW[4096];
+		/* We need to put the text in a seperate buffer because the default
+		 * buffer in MB_TO_WIDE might not be large enough (512 chars) */
+		wchar_t help_msgW[8192];
 #endif
 		ReleaseCapture();
-		_left_button_clicked =_left_button_down = false;
+		_left_button_clicked = _left_button_down = false;
 
 		old = MyShowCursor(true);
 		if (MessageBox(GetActiveWindow(), MB_TO_WIDE_BUFFER(str, help_msgW, lengthof(help_msgW)), _T("OpenTTD"), MB_ICONINFORMATION | MB_OKCANCEL) == IDCANCEL) {
@@ -938,15 +959,6 @@ void ShowInfo(const char *str)
 	}
 }
 
-#ifdef __MINGW32__
-	/* _set_error_mode() constants&function (do not exist in mingw headers) */
-	#define _OUT_TO_DEFAULT      0
-	#define _OUT_TO_STDERR       1
-	#define _OUT_TO_MSGBOX       2
-	#define _REPORT_ERRMODE      3
-	int _set_error_mode(int);
-#endif
-
 #if defined(WINCE)
 int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
 #else
@@ -965,7 +977,7 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 
 #if !defined(WINCE)
 	/* Check if a win9x user started the win32 version */
-	if (HasBit(GetVersion(), 31)) error("This version of OpenTTD doesn't run on windows 95/98/ME.\nPlease download the win9x binary and try again.");
+	if (HasBit(GetVersion(), 31)) usererror("This version of OpenTTD doesn't run on windows 95/98/ME.\nPlease download the win9x binary and try again.");
 #endif
 
 	/* For UNICODE we need to convert the commandline to char* _AND_
@@ -1024,7 +1036,7 @@ void GetCurrentDirectoryW(int length, wchar_t *path)
 char *getcwd(char *buf, size_t size)
 {
 #if defined(WINCE)
- 	TCHAR path[MAX_PATH];
+	TCHAR path[MAX_PATH];
 	GetModuleFileName(NULL, path, MAX_PATH);
 	convert_from_fs(path, buf, size);
 	/* GetModuleFileName returns dir with file, so remove everything behind latest '\\' */
@@ -1043,7 +1055,6 @@ char *getcwd(char *buf, size_t size)
 
 void DetermineBasePaths(const char *exe)
 {
-	extern void ScanForTarFiles();
 	char tmp[MAX_PATH];
 	TCHAR path[MAX_PATH];
 #ifdef WITH_PERSONAL_DIR
@@ -1071,13 +1082,13 @@ void DetermineBasePaths(const char *exe)
 	_searchpaths[SP_WORKING_DIR] = strdup(tmp);
 
 	if (!GetModuleFileName(NULL, path, lengthof(path))) {
-		DEBUG(misc, 0, "GetModuleFileName failed (%d)\n", GetLastError());
+		DEBUG(misc, 0, "GetModuleFileName failed (%lu)\n", GetLastError());
 		_searchpaths[SP_BINARY_DIR] = NULL;
 	} else {
 		TCHAR exec_dir[MAX_PATH];
 		_tcsncpy(path, MB_TO_WIDE_BUFFER(exe, path, lengthof(path)), lengthof(path));
 		if (!GetFullPathName(path, lengthof(exec_dir), exec_dir, NULL)) {
-			DEBUG(misc, 0, "GetFullPathName failed (%d)\n", GetLastError());
+			DEBUG(misc, 0, "GetFullPathName failed (%lu)\n", GetLastError());
 			_searchpaths[SP_BINARY_DIR] = NULL;
 		} else {
 			strecpy(tmp, WIDE_TO_MB_BUFFER(exec_dir, tmp, lengthof(tmp)), lastof(tmp));
@@ -1089,8 +1100,6 @@ void DetermineBasePaths(const char *exe)
 
 	_searchpaths[SP_INSTALLATION_DIR]       = NULL;
 	_searchpaths[SP_APPLICATION_BUNDLE_DIR] = NULL;
-
-	ScanForTarFiles();
 }
 
 /**
@@ -1125,7 +1134,7 @@ bool InsertTextBufferClipboard(Textbuf *tb)
 		cbuf = GetClipboardData(CF_TEXT);
 
 		ptr = (const char*)GlobalLock(cbuf);
-		ttd_strlcpy(utf8_buf, FS2OTTD(ptr), lengthof(utf8_buf));
+		strecpy(utf8_buf, FS2OTTD(ptr), lastof(utf8_buf));
 
 		GlobalUnlock(cbuf);
 		CloseClipboard();
@@ -1139,8 +1148,8 @@ bool InsertTextBufferClipboard(Textbuf *tb)
 	for (ptr = utf8_buf; (c = Utf8Consume(&ptr)) != '\0';) {
 		if (!IsPrintable(c)) break;
 
-		size_t len = Utf8CharLen(c);
-		if (tb->length + length >= tb->maxlength - (uint16)len) break;
+		byte len = Utf8CharLen(c);
+		if (tb->size + length + len > tb->maxsize) break;
 
 		byte charwidth = GetCharacterWidth(FS_NORMAL, c);
 		if (tb->maxwidth != 0 && width + tb->width + charwidth > tb->maxwidth) break;
@@ -1151,15 +1160,15 @@ bool InsertTextBufferClipboard(Textbuf *tb)
 
 	if (length == 0) return false;
 
-	memmove(tb->buf + tb->caretpos + length, tb->buf + tb->caretpos, tb->length - tb->caretpos);
+	memmove(tb->buf + tb->caretpos + length, tb->buf + tb->caretpos, tb->size - tb->caretpos);
 	memcpy(tb->buf + tb->caretpos, utf8_buf, length);
 	tb->width += width;
 	tb->caretxoffs += width;
 
-	tb->length += length;
+	tb->size += length;
 	tb->caretpos += length;
-	assert(tb->length < tb->maxlength);
-	tb->buf[tb->length] = '\0'; // terminating zero
+	assert(tb->size <= tb->maxsize);
+	tb->buf[tb->size - 1] = '\0'; // terminating zero
 
 	return true;
 }
@@ -1210,7 +1219,7 @@ const char *FS2OTTD(const TCHAR *name)
 		wchar_t w;
 		int len = MultiByteToWideChar(_codepage, 0, name, 1, &w, 1);
 		if (len != 1) {
-			DEBUG(misc, 0, "[utf8] M2W error converting '%c'. Errno %d", *name, GetLastError());
+			DEBUG(misc, 0, "[utf8] M2W error converting '%c'. Errno %lu", *name, GetLastError());
 			continue;
 		}
 
@@ -1249,7 +1258,7 @@ const TCHAR *OTTD2FS(const char *name)
 		char mb;
 		int len = WideCharToMultiByte(_codepage, 0, (wchar_t*)&c, 1, &mb, 1, NULL, NULL);
 		if (len != 1) {
-			DEBUG(misc, 0, "[utf8] W2M error converting '0x%X'. Errno %d", c, GetLastError());
+			DEBUG(misc, 0, "[utf8] W2M error converting '0x%X'. Errno %lu", c, GetLastError());
 			continue;
 		}
 
@@ -1270,9 +1279,9 @@ const TCHAR *OTTD2FS(const char *name)
  * @return pointer to utf8_buf. If conversion fails the string is of zero-length */
 char *convert_from_fs(const wchar_t *name, char *utf8_buf, size_t buflen)
 {
-	int len = WideCharToMultiByte(CP_UTF8, 0, name, -1, utf8_buf, buflen, NULL, NULL);
+	int len = WideCharToMultiByte(CP_UTF8, 0, name, -1, utf8_buf, (int)buflen, NULL, NULL);
 	if (len == 0) {
-		DEBUG(misc, 0, "[utf8] W2M error converting wide-string. Errno %d", GetLastError());
+		DEBUG(misc, 0, "[utf8] W2M error converting wide-string. Errno %lu", GetLastError());
 		utf8_buf[0] = '\0';
 	}
 
@@ -1289,9 +1298,9 @@ char *convert_from_fs(const wchar_t *name, char *utf8_buf, size_t buflen)
  * @return pointer to utf16_buf. If conversion fails the string is of zero-length */
 wchar_t *convert_to_fs(const char *name, wchar_t *utf16_buf, size_t buflen)
 {
-	int len = MultiByteToWideChar(CP_UTF8, 0, name, -1, utf16_buf, buflen);
+	int len = MultiByteToWideChar(CP_UTF8, 0, name, -1, utf16_buf, (int)buflen);
 	if (len == 0) {
-		DEBUG(misc, 0, "[utf8] M2W error converting '%s'. Errno %d", name, GetLastError());
+		DEBUG(misc, 0, "[utf8] M2W error converting '%s'. Errno %lu", name, GetLastError());
 		utf16_buf[0] = '\0';
 	}
 
@@ -1333,7 +1342,7 @@ HRESULT OTTDSHGetFolderPath(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,
 	{
 		DWORD ret;
 		switch (csidl) {
-			case CSIDL_FONTS: /* Get the system font path, eg %WINDIR%\Fonts */
+			case CSIDL_FONTS: // Get the system font path, eg %WINDIR%\Fonts
 				ret = GetEnvironmentVariable(_T("WINDIR"), pszPath, MAX_PATH);
 				if (ret == 0) break;
 				_tcsncat(pszPath, _T("\\Fonts"), MAX_PATH);
